#!/bin/bash

case $1 in
  -pre|-patch) PATCH_ARG=$1 ;;
  *) PATCH_ARG= ;;
esac

umask 022

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

# -----------------------------------------------------------------------------
# show environment
# -----------------------------------------------------------------------------

show_env () {
  # usage: show_env line_number var1 var2 ...
  [[ -z $VERBOSE ]] && return
  # first argument $1 is line_number
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  # other arguments are envvars, show name and value
  for i in $@ ; do
    if [[ -n $(eval echo '${!'$i'*}') ]] ; then
      echo "$LINE_NUMBER: $i=$(eval echo '$'$i)"
    else
      echo "$LINE_NUMBER: $i=(unset)"
    fi
  done
}

# -----------------------------------------------------------------------------
# set environment
# -----------------------------------------------------------------------------

EPAGES_INSTALL_LANGUAGES=${EPAGES_LANGUAGES:-$EPAGES_INSTALL_LANGUAGES}
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
[[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6
EPAGES_INSTALL_LANGUAGES=${EPAGES_INSTALL_LANGUAGES:-$EPAGES_LANGUAGES}

if [[ -n $EPAGES_INSTALL_LANGUAGES ]] ; then
  # add default languages unless already defined
  export EPAGES_LANGUAGES=$EPAGES_INSTALL_LANGUAGES
  [[ $EPAGES_LANGUAGES =~ de ]] || EPAGES_LANGUAGES="de $EPAGES_LANGUAGES"
  [[ $EPAGES_LANGUAGES =~ en ]] || EPAGES_LANGUAGES="en $EPAGES_LANGUAGES"
fi

EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

. /etc/default/$EPAGES_PROJECT
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

NEW_INSTALLATION=1
# installation is old if has Cartridges.d/*.conf with DE_EPAGES::Object in it
if grep -q '^DE_EPAGES::Object=' "$EPAGES_CONFIG"/Cartridges.d/*.conf 2>/dev/null ; then
  NEW_INSTALLATION=
fi

# -----------------------------------------------------------------------------
# disable epages-update in distributed environment
# -----------------------------------------------------------------------------

if [[ $EPAGES_DISTRIBUTED_INSTALLATION = 1 ]] ; then
  ini-config.sh -f /etc/yum.repos.d/epages-update.repo -s epages-update -k enabled -v 0 -i
fi

# -----------------------------------------------------------------------------
# install or start release
# -----------------------------------------------------------------------------

show_env $LINENO NEW_INSTALLATION PATCH_ARG EPAGES_DO_PATCH

if [[ $EPAGES_DO_PATCH != 0 ]] ; then
  # if initial installation (script has no argument -pre/-patch)
  if [[ -z $PATCH_ARG ]] ; then
    # then start epages (and install it unless already installed)
    /etc/init.d/$EPAGES_PROJECT start || exit $?
    start_epagesj_if_stopped || exit $?
  # if patch installation and function start_epages_if_stopped exists
  elif [[ $PATCH_ARG = -patch ]] && type -t start_epages_if_stopped >/dev/null ; then
    # then start each stopped epages service
    start_epagesj_if_stopped || exit $?
    start_epages_if_stopped || exit $?
  fi
fi

# -----------------------------------------------------------------------------
# find current version
# -----------------------------------------------------------------------------

if ls "$EPAGES_CONFIG"/Cartridges.d/*.conf >/dev/null 2>&1 ; then
  CONFFILE="$EPAGES_CONFIG"/Cartridges.d/*.conf
else
  # incomplete epages installation
  exit 0
fi

CURRENT_VERSION=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILE)
export EPAGES_VALID_VERSION=$CURRENT_VERSION

show_env $LINENO CONFFILE CURRENT_VERSION

# -----------------------------------------------------------------------------
# disable epages-setup.repo
# -----------------------------------------------------------------------------

if [ -f /etc/yum.repos.d/epages-setup.repo ] ; then
  ini-config.sh -f /etc/yum.repos.d/epages-setup.repo -s epages-setup -k enabled -v 0 -i
fi

# -----------------------------------------------------------------------------
# find patch version
# -----------------------------------------------------------------------------

EPAGES_PATCHES=${EPAGES_PATCHES:-"$EPAGES_SHARED"/Patches}
# sort patch_[0-9]*_[0-9]*_*[0-9] by version (-v) and get highest (1st line) 
PATCH_TAG=$(ls -dv "$EPAGES_PATCHES"/patch_[0-9]*_[0-9]*_*[0-9] 2>/dev/null | tail -1)
# get basename of PATCH_TAG (patch_X_Y_Z)
PATCH_TAG=${PATCH_TAG##*/}

# if Service/epagesService exists (i.e. single/alpha host), then install patch
# otherwise just run preinstall
[ -f "$EPAGES"/Service/epagesService ] && INSTALL=install || INSTALL=preinstall

# -----------------------------------------------------------------------------
# check prerequisits
# -----------------------------------------------------------------------------

show_env $LINENO PATCH_ARG EPAGES_PATCHES INSTALL EPAGES_DO_PATCH PATCH_TAG

cd "$EPAGES_PATCHES"/$PATCH_TAG 2>/dev/null

# if script is executed with -pre then just check prerequisits
if [[ $PATCH_ARG = -pre && -x ./patch.sh ]] ; then
  # exit if no patch dir found
  [[ -n $PATCH_TAG ]] || exit 0
  # exit if not single/alpha host
  [[ $INSTALL = install ]] || exit 0
  # exit if EPAGES_DO_PATCH=0 is set outside script
  [[ $EPAGES_DO_PATCH = 0 ]] && exit 0

  echo "-> ./patch.sh get_var"
  ./patch.sh get_var
  RESULT=$?

  echo "./patch.sh get_var: $RESULT" >> /tmp/epages-$PATCH_TAG-result.log
  [[ $RESULT = 0 ]] && echo '-> SUCCESS' || { echo '-> FAILED'; exit 1; }

  exit 0
fi

# -----------------------------------------------------------------------------
# run patch?
# -----------------------------------------------------------------------------

if [[ -n $PATCH_TAG && -z ${!EPAGES_DO_PATCH*} ]] ; then
  # find own ip addresses
  IFCONFIG=$(ip addr show | awk '/ inet6? /{sub(/\/.*/,"",$2);printf "%s ", $2}' 2>/dev/null)

  # check if SystemDomainName in epages.conf belongs to own addresses
  # if not, do not start ./patch.sh install
  $PERL -e '
use Sys::Hostname;
use Sys::HostIP;
use IO::Socket;
use DE_EPAGES::Core::API::EpagesConfig qw(EpagesConfig);
use DE_EPAGES::WebInterface::API::Config qw(WebInterfaceConfig);
use DE_EPAGES::Core::API::PerlTools qw(GetIPAddresses);

my $IPs = Sys::HostIP->ips;
push @$IPs, inet_ntoa( scalar( gethostbyname( hostname())));
push @$IPs, "::1";
push @$IPs, @{GetIPAddresses()->{6}};
push @$IPs, @ARGV;

my $SystemAddr = EpagesConfig->get("DE_EPAGES::Object", "SystemDomainName", "localhost");
$SystemAddr = inet_ntoa(scalar( gethostbyname($SystemAddr) ))
    unless $SystemAddr =~ /^[0-9a-fA-F:.]+$/;
foreach my $TestIP ($SystemAddr, @{WebInterfaceConfig->sectionNames}) {
    next unless $TestIP =~ /^[0-9a-fA-F:.]+$/;
    exit 1 unless $TestIP ~~ @$IPs;
}
' $IFCONFIG >/dev/null 2>&1 || EPAGES_DO_PATCH=0
fi

# -----------------------------------------------------------------------------
# install patch
# -----------------------------------------------------------------------------

export IGNORE_RUNNING_JOBS=1

show_env $LINENO IGNORE_RUNNING_JOBS IFCONFIG

if [[ -n $PATCH_TAG && $EPAGES_DO_PATCH = 0 && -x ./patch.sh ]] ; then
  # if patch dir found but EPAGES_DO_PATCH=0 just remember
  echo "Now run: cd \$EPAGES_PATCHES/$PATCH_TAG && ./patch.sh $INSTALL"
elif [[ -n $PATCH_TAG && -x ./patch.sh ]] ; then
  # elif patch dir found then run patch
  echo "-> ./patch.sh $INSTALL"
  ./patch.sh $INSTALL
  RESULT=$?
  echo "./patch.sh $INSTALL: $RESULT" >> /tmp/epages-$PATCH_TAG-result.log
  [[ $RESULT = 0 ]] && echo '-> SUCCESS' || { echo '-> FAILED'; exit 1; }
fi

show_env $LINENO EPAGES_DO_PATCH NEW_INSTALLATION EPAGES_INSTALL_LANGUAGES

# if EPAGES_DO_PATCH=0 or old installation found or epages languages shall not be installed
# then exit
[[ $EPAGES_DO_PATCH = 0 || -z $NEW_INSTALLATION || -z $EPAGES_INSTALL_LANGUAGES ]] && exit 0

show_env $LINENO PERL EPAGES_CARTRIDGES EPAGES_LANGUAGES EPAGES_INSTALL_LANGUAGES

# add languages as described in EPAGES_INSTALL_LANGUAGES
ADD_LANGUAGES="$EPAGES_CARTRIDGES"/DE_EPAGES/Installer/Scripts/addLanguages.pl
$PERL "$ADD_LANGUAGES" -lang "$EPAGES_INSTALL_LANGUAGES"
RESULT=$?

if [[ -f /tmp/epages-$PATCH_TAG-result.log ]] ; then
  echo "$PERL $ADD_LANGUAGES -lang '$EPAGES_INSTALL_LANGUAGES': $RESULT" \
  >> /tmp/epages-$PATCH_TAG-result.log
fi

exit $RESULT
