#!/bin/bash

# -----------------------------------------------------------------
# variables
# -----------------------------------------------------------------

# sourcing /etc/default/epages6
. /etc/default/epages6
# provides following variables/functions:
export test_version

BN=${0##*/}
MONGO_CMD=mongo

# -----------------------------------------------------------------
# arguments
# -----------------------------------------------------------------

usage() {
  [[ $# -gt 0 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $0 -d dbname -u dbuser -p password -r role"
  echo "Description:"
  echo "  add/create user in a mongodb"
  echo "Options:"
  echo "  -d : name of the database"
  echo "  -u : name of the user to create"
  echo "  -p : password for the created user"
  echo "  -r : comma-sep. list of roles for the user,"
  echo "       see http://docs.mongodb.org/manual/reference/built-in-roles"
  echo "Example:"
  echo "  $0 -d mongo24 -u admin -p secret -r dbAdmin,userAdmin,readWrite"
  echo "  $0 -d mongo26 -u admin -p secret -r dbOwner"
  exit 1
}

#get options from command line
unset DB_NAME DB_USER DB_PW DB_ROLES
while getopts d:u:p:r: char; do
case $char in
    d) DB_NAME=$OPTARG ;;
    u) DB_USER=$OPTARG ;;
    p) DB_PW=$OPTARG ;;
    r) DB_ROLES=$OPTARG ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`

[[ -z $DB_NAME ]] && usage "$BN: Option -d missing"
[[ -z $DB_USER ]] && usage "$BN: Option -u missing"
[[ -z $DB_PW ]] && usage "$BN: Option -p missing"
[[ -z $DB_ROLES ]] && usage "$BN: Option -r missing"
[[ $DB_ROLES = ${DB_ROLES// /} ]] \
|| usage "$BN: Option -r must not contain spaces"
[[ $# -ne 0 ]] && usage "$BN: No extra arguments allowed: $@"

#get mongo version or die
MONGO_VERSION=$($MONGO_CMD --version) || exit 1
MONGO_VERSION=${MONGO_VERSION##*:}

# -----------------------------------------------------------------
# create user by either addUser (mongo 2.4) or createUser (mongo 2.6)
# -----------------------------------------------------------------

# quote arguments
DB_USER="\"$DB_USER\""
DB_PW="\"$DB_PW\""
DB_ROLES="\"${DB_ROLES//,/\",\"}\""

if test_version $MONGO_VERSION -lt 2.6 ; then
  STATEMENT="db.addUser({user:$DB_USER, pwd:$DB_PW, roles:[$DB_ROLES]})"
else
  STATEMENT="db.createUser({user:$DB_USER, pwd:$DB_PW, roles:[{role:$DB_ROLES, db:\"$DB_NAME\"}]})"
fi

# -----------------------------------------------------------------
# wait until mongod has started
# -----------------------------------------------------------------

RESULT=1 ; COUNT=0
while [[ $COUNT -lt 5 ]] ; do
  COUNT=$(($COUNT + 1))
  echo "$STATEMENT" | $MONGO_CMD --quiet $DB_NAME 2>/tmp/$USER-mongo_user.err
  RESULT=$?
  if grep -q . /tmp/$USER-mongo_user.err ; then
    RESULT=1
    # break loop if other error than 'connect failed'
    grep -q 'connect failed' /tmp/$USER-mongo_user.err || break
  fi
  [[ $RESULT = 0 ]] && break
  sleep 8
done

exit $RESULT
