#!/bin/bash

# -----------------------------------------------------------------
# variables
# -----------------------------------------------------------------

# sourcing /etc/default/epages6
. /etc/default/epages6
# provides following variables/functions:
export MONGOD_CONF EPAGESJ EPAGESJ_CONFIG
export mongod_name test_version

BN=${0##*/}
DN=$(cd $(dirname $0) && pwd)
OPENSSL_CMD=openssl
MONGO_CMD=mongo
INI_CONFIG=/var/epages/ini-config.sh

# used scripts
CREATE_USER_SH="$DN/mongo_create_user.sh"
HAS_USER_SH="$DN/mongo_has_user.sh"
KEYFILE_SH="$DN/mongo_keyfile.sh"
UPDATE_PASSWORD_SH="$DN/mongo_update_password.sh"

# mongo daemon name (used for stop/start)
MONGOD=$(mongod_name)

# argument defaults
MONGO_CONF_FILE=${MONGOD_CONF:-"/etc/$MONGOD.conf"}
EPAGESJ=${EPAGESJ:-$EPAGES/j}
EPAGESJ_CONF=${EPAGESJ_CONFIG:-$EPAGESJ/conf}/epagesj.conf

# -----------------------------------------------------------------
# arguments
# -----------------------------------------------------------------

usage() {
  [[ $# -gt 0 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $0 [-f keyfile] [-c mongodb.conf] [-j epagesj.conf] [-h] [-v]"
  echo "Description:"
  echo "  enable mongodb for authorization by creating db users"
  echo "  and configure them in a keyfile to use"
  echo "Options:"
  echo "  -f: auth keyfile to create and use (default: <dbpath>/mongodb-keyfile)"
  echo "  -c: mongodb config file (default: $MONGO_CONF_FILE)"
  echo "  -j: epagesj config file (default: $EPAGESJ_CONF)"
  echo "  -h: show usage"
  echo "  -v: verbose"
  echo "Example:"
  echo "  $0 -f /usr/local/etc/mongo_secret -c /etc/mongo_xyz.conf"
  exit 1
}

#get options from command line
unset KEYFILE VERBOSE
while getopts f:c:j:v char; do
case $char in
    f) KEYFILE=$OPTARG ;;
    c) MONGO_CONF_FILE=$OPTARG ;;
    j) EPAGESJ_CONF=$OPTARG ;;
    v) VERBOSE=1 ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`

[[ $# -ne 0 ]] && usage "$BN: No extra arguments allowed: $@"
[[ -f "$MONGO_CONF_FILE" ]] || usage "$BN: No such file: $MONGO_CONF_FILE"
[[ -f "$EPAGESJ_CONF" ]] || usage "$BN: No such file: $EPAGESJ_CONF"

#check if openssl is installed
type $OPENSSL_CMD >/dev/null || exit 1

#get mongo version or die
MONGO_VERSION=$($MONGO_CMD --version) || exit 1
MONGO_VERSION=${MONGO_VERSION##*:}

#how to enable auth:
#http://docs.mongodb.org/manual/tutorial/enable-authentication-without-bypass/

# -----------------------------------------------------------------
# get mongodb information from epagesj.conf
# -----------------------------------------------------------------

unset new_password new_user new_database
user=$($INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k username -G)
[[ -z $user ]] && { user=epagesj ; new_user=epagesj; }

password=$($INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k password -G)
# epagesj does not like [+/] in mongo passwords
[[ -z $password ]] && new_password=$($OPENSSL_CMD rand 6 -base64 | tr +/ 00)
password=${password:-$new_password}

database=$($INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k databaseName -G)
[[ -z $database ]] && { database=epagesj ; new_database=epagesj; }

# -----------------------------------------------------------------
# restart mongodb without authentication if client request fails
# -----------------------------------------------------------------

unset MONGO_CONF_CHANGED
ERROR_MSG=$($HAS_USER_SH -d admin -u siteRootAdmin 2>&1 >/dev/null)

if [[ -n $ERROR_MSG ]] ; then
  service $MONGOD stop
  cp -p $MONGO_CONF_FILE $MONGO_CONF_FILE.change_auth
  egrep -v '^[[:space:]]*(auth|keyFile)[[:space:]]*=' $MONGO_CONF_FILE.change_auth > $MONGO_CONF_FILE
  MONGO_CONF_CHANGED=1
  service $MONGOD start
  sleep 1
fi

# -----------------------------------------------------------------
# create mongodb users (or remember existing users)
# -----------------------------------------------------------------

# mongo 2.6 introduces some abbreviations for roles:
# root is dbAdminAnyDatabase,userAdminAnyDatabase,clusterAdmin,readWriteAnyDatabase
# dbOwner is dbAdmin,userAdmin,readWrite
# see http://docs.mongodb.org/manual/reference/built-in-roles
declare -A ROLE
if test_version $MONGO_VERSION -lt 2.6 ; then
  ROLE[siteUserAdmin]=userAdminAnyDatabase
  ROLE[siteRootAdmin]=dbAdminAnyDatabase,userAdminAnyDatabase,clusterAdmin,readWriteAnyDatabase
  ROLE[dbOwner]=dbAdmin,userAdmin,readWrite
else
  ROLE[siteUserAdmin]=userAdminAnyDatabase
  ROLE[siteRootAdmin]=root
  ROLE[dbOwner]=dbOwner
fi

declare -A EXISTING

if $HAS_USER_SH -d admin -u siteRootAdmin 2>/dev/null ; then
  EXISTING[siteRootAdmin]=admin
else
  [[ -n $VERBOSE ]] && echo "$CREATE_USER_SH -d admin -u siteRootAdmin -p '...' -r ${ROLE[siteRootAdmin]}"
  $CREATE_USER_SH -d admin -u siteRootAdmin -p "$password" -r ${ROLE[siteRootAdmin]}
fi

if $HAS_USER_SH -d admin -u siteUserAdmin 2>/dev/null ; then
  EXISTING[siteUserAdmin]=admin
else
  [[ -n $VERBOSE ]] && echo "$CREATE_USER_SH -d admin -u siteUserAdmin -p '...' -r ${ROLE[siteUserAdmin]}"
  $CREATE_USER_SH -d admin -u siteUserAdmin -p "$password" -r ${ROLE[siteUserAdmin]}
fi

if $HAS_USER_SH -d "$database" -u "$user" 2>/dev/null ; then
  EXISTING[$user]=$database
else
  [[ -n $VERBOSE ]] && echo "$CREATE_USER_SH -d '$database' -u '$user' -p '...' -r ${ROLE[dbOwner]}"
  $CREATE_USER_SH -d "$database" -u "$user" -p "$password" -r ${ROLE[dbOwner]}
fi

# -----------------------------------------------------------------
# update password for existing users
# -----------------------------------------------------------------

unset USER_FAILURE
for name in ${!EXISTING[@]} ; do
  [[ -n $VERBOSE ]] && echo "$UPDATE_PASSWORD_SH -d '${EXISTING[$name]}' -u '$name' -p '...'"
  $UPDATE_PASSWORD_SH -d "${EXISTING[$name]}" -u "$name" -p "$password" || USER_FAILURE=1
done

# -----------------------------------------------------------------
# restore mongo.conf if changed
# -----------------------------------------------------------------

[[ -n $MONGO_CONF_CHANGED ]] && mv $MONGO_CONF_FILE.change_auth $MONGO_CONF_FILE

# -----------------------------------------------------------------
# create keyfile and change mongo.conf
# -----------------------------------------------------------------

if [[ -z $USER_FAILURE ]] ; then
  [[ -n $VERBOSE ]] && echo "$KEYFILE_SH${KEYFILE:+ -f $KEYFILE} -c $MONGO_CONF_FILE"
  $KEYFILE_SH ${KEYFILE:+-f $KEYFILE} -c $MONGO_CONF_FILE || exit 1
fi

# -----------------------------------------------------------------
# restart mongodb with authentication
# -----------------------------------------------------------------

service $MONGOD restart || exit 1
[[ -n $USER_FAILURE ]] && exit 1
sleep 1

# -----------------------------------------------------------------
# wait until mongod has started
# -----------------------------------------------------------------

COUNT=0
while [[ $COUNT -lt 5 ]] ; do
  COUNT=$(($COUNT + 1))
  $HAS_USER_SH -d admin -u siteUserAdmin 2>/tmp/$USER-mongo_auth.err
  grep -q 'connect failed' /tmp/$USER-mongo_auth.err || break
  sleep 8
done

# -----------------------------------------------------------------
# check that all users exist
# -----------------------------------------------------------------

EXISTING[siteRootAdmin]=admin
EXISTING[siteUserAdmin]=admin
EXISTING[$user]=$database

for name in ${!EXISTING[@]} ; do
  [[ -n $VERBOSE ]] && echo "$MONGO_CMD '${EXISTING[$name]}' --quiet -u '$name' -p '...'"
  echo exit | $MONGO_CMD "${EXISTING[$name]}" --quiet -u "$name" -p "$password" || USER_FAILURE=1
done
[[ -n $USER_FAILURE ]] && exit 1

# -----------------------------------------------------------------
# update mongodb information on epagesj.conf
# -----------------------------------------------------------------

if [[ -n $new_user ]] ; then
  [[ -n $VERBOSE ]] && echo "$INI_CONFIG -f '$EPAGESJ_CONF' -s Mongo -k username -v '$user' -i"
  $INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k username -v "$user" -i || exit 1
fi

if [[ -n $new_password ]] ; then
  [[ -n $VERBOSE ]] && echo "$INI_CONFIG -f '$EPAGESJ_CONF' -s Mongo -k password -v '...' -i"
  $INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k password -v "$password" -i || exit 1
fi

if [[ -n $new_database ]] ; then
  [[ -n $VERBOSE ]] && echo "$INI_CONFIG -f '$EPAGESJ_CONF' -s Mongo -k databaseName -v '$database' -i"
  $INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k databaseName -v "$database" -i || exit 1
fi

# -----------------------------------------------------------------
# remember user to restart epagesj service
# -----------------------------------------------------------------

if [[ -n "$new_user$new_password$new_database" ]] ; then
  echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
  echo "Info: $MONGOD authentication has changed."
  echo "Therefore run: service epagesj restart"
  echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
fi

exit ${USER_FAILURE:-0}
