#!/bin/bash

#==============================================================================
# auxiliary functions for epages shell scripts
#==============================================================================

if [[ -z $functions_sourced ]] ; then

functions_sourced=1

umask 027 # drwxr-x---

# architecture

[[ -n $fun_arch_sourced ]] || . ${BASH_SOURCE%.sh}/fun-arch.sh

# some OS use libxslt.so.1 / libexslt.so.0 / libxml2.so.2
# compiled by epages stored in $EPAGES_PERL/lib32/$EPAGES_KERNEL

EPAGES_PERL=${EPAGES_PERL:-/srv/epages/eproot/Perl}
if [[ -w /etc/passwd ]] ; then
  for i in libxslt.so.1 libexslt.so.0 libxml2.so.2 ; do
    [[ -f $EPAGES_PERL/lib32/$EPAGES_KERNEL/$i ]] || continue
    for j in $(ls -d /usr/lib/$i{,.*} 2>/dev/null) ; do
      [[ -f $j ]] || continue
      [[ -e /lib/${j##*/} ]] || cp -pP $j /lib
    done
    [[ -f /usr/lib/$i && -f "$(readlink -qf /lib/$i)" ]] && rm -f /usr/lib/$i
  done
fi

# configuration

[[ -n $config_sourced ]] || . ${BASH_SOURCE%.sh}/config.sh

# terminal

[[ -n $fun_term_sourced ]] || . ${BASH_SOURCE%.sh}/fun-term.sh

# basic linux

[[ -n $fun_linux_sourced ]] || . ${BASH_SOURCE%.sh}/fun-linux.sh

# timeout_test

export timeout_test
timeout_test() {
  if [[ $# -le 1 ]] ; then
    echo 'timeout_test FILE_TEST_OPERATOR TEST_OBJECT'
    echo "FILE_TEST_OPERATOR: see 'man test', e.g. one of: -f -d -e -x"
    echo "TEST_OBJECT: file/directory to test"
    return 1
  fi
  local HOW=$1 ; shift
  local TEST=/bin/test
  [ -x /usr/bin/test ] && TEST=/usr/bin/test
  /var/epages/exec.sh -c "$TEST $HOW '$@'"
  return $?
}

[[ $EPAGES_TEST = timeout_test ]] && alias test=timeout_test

# ipcrm_user

export ipcrm_user
ipcrm_user() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'ipcrm_user USER'
    echo "USER: run ipcrm for UNIX user"
    return 1
  fi
  local user=$1
  [ -z "`ps -u$user -opid= 2>/dev/null`" ] || return
  ipcs -s >/dev/null 2>&1  || return

  ipcs -s | grep $user | while read T ID ELSE ; do ipcrm -s $ID ; done
  ipcs -m | grep $user | while read T ID ELSE ; do ipcrm -m $ID ; done
}

# processes

[[ -n $fun_process_sourced ]] || . ${BASH_SOURCE%.sh}/fun-process.sh

# run scripts

[[ -n $fun_runscript_sourced ]] || . ${BASH_SOURCE%.sh}/fun-runscript.sh

# started_by_rpm

export started_by_rpm
started_by_rpm() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'started_by_rpm PID'
    echo "returns: 0 if one of ancestors of PID is rpm/yum, else 1"
    return 1
  fi

  ( get_pidof yum ; get_pidof zypp ; get_pidof zypper
    get_pidof apt-get ; get_pidof dpkg ) | grep -q '[0-9]' && return 0
  get_pidfile -p /var/run/yum.pid && return 0
  get_pidfile -p /var/run/zypp.pid && return 0

  USER=${LOGNAME:-$USER}
  ps xwww --forest | tac > /tmp/$USER-started_by_rpm.0
  sed -rn "/^\s+$1\s+/,\$p" /tmp/$USER-started_by_rpm.0 > /tmp/$USER-started_by_rpm.1
  echo PID=$1 >> /tmp/$USER-started_by_rpm.1
  while read -r pid tty stat time cmd ; do
    [[ "$cmd" != "${cmd#*started_by_rpm}" ]] && continue
    [[ "$cmd" != "${cmd#*rpm}" ]] && return 0
    [[ "$cmd" = "${cmd#*\_}" ]] && break
  done < /tmp/$USER-started_by_rpm.1
  return 1
}

# java functions

[[ -n $fun_java_sourced ]] || . ${BASH_SOURCE%.sh}/fun-java.sh

# epages functions

[[ -n $fun_epages_sourced ]] || . ${BASH_SOURCE%.sh}/fun-epages.sh

# MySQL functions

[[ -n $fun_mysql_sourced ]] || . ${BASH_SOURCE%.sh}/fun-mysql.sh

# httpd functions

[[ -n $fun_httpd_sourced ]] || . ${BASH_SOURCE%.sh}/fun-httpd.sh

# check nfs client

export check_nfs
check_nfs() {
  if [[ $# -gt 0 ]] ; then
    echo 'check_nfs'
    echo "returns: 0 if NFS client process rpc.statd is running, else 1"
    return 1
  fi
  ps -Crpc.statd || { echo "rpc.statd: no such process"; return 1; }
  return 0
}

# perl environment

export epages_perl5lib
epages_perl5lib() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_perl5lib'
    echo 'echoes PERL5LIB for epages perl'
    return 1
  fi
  local ELIB="$EPAGES_PERL/lib"
  echo "$EPAGES_CARTRIDGES:$ELIB/site_perl/linux:$ELIB/site_perl:$ELIB/linux:$ELIB"
}

# path

PATH=$(pathmunge "/bin:/usr/bin:/sbin:/usr/sbin" "$PATH")
PATH=$(pathmunge "$PATH:$EPAGES_PERL/bin:$HTTPD_ROOT/bin")
export PATH

export epages_path
epages_path() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_path'
    echo "prepends PATH with EPAGES_PERL/bin"
    return 1
  fi
  pathmunge "$EPAGES_PERL/bin:$PATH"
}

# libraries

export LD_LIBRARY_PATH
export epages_libs
epages_libs() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_libs'
    echo 'prepends LD_LIBRARY_PATH with epages libdirs'
    return 1
  fi
  pathmunge "$EPAGES_PERL/lib32/$EPAGES_KERNEL:$EPAGES_PERL/lib32/ssl:$EPAGES_PERL/bin:$HTTPD_ROOT/lib:/lib32:/lib:/usr/lib:/usr/X11R6/lib:$LD_LIBRARY_PATH:/usr/lib32:/usr/lib/i386-linux-gnu:/lib/i386-linux-gnu"
}

# selinux

export set_selinux4epages
set_selinux4epages() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'set_selinux4epages'
    echo "sets SELinux file context for ePages files (if EPAGES_ENABLE_SELINUX is set)"
    return 1
  fi
  [[ ${EPAGES_ENABLE_SELINUX:-0} != 0 ]] || return
  test -x /var/epages/se-file-context.sh || return
  [[ ${EPAGES_ENABLE_PERL_CGI:-0} = 0 ]] && unset PERL_CGI || PERL_CGI=--perl-cgi
  /var/epages/se-file-context.sh --set $PERL_CGI
}

# user/group

[[ -n $fun_account_sourced ]] || . ${BASH_SOURCE%.sh}/fun-account.sh

# host name and IP address

[[ -n $fun_host_sourced ]] || . ${BASH_SOURCE%.sh}/fun-host.sh

# copy_ini_values

export copy_ini_values
copy_ini_values () {
  if [[ $# -lt 2 ]] || ! ls "$1" "$2" >/dev/null ; then
     echo 'copy_ini_values SOURCE_INI TARGET_INI'
     echo 'copies all values from SOURCE_INI to TARGET_INI'
     return 1
  fi
  grep -v '^[[:space:]]*[;#]' "$1" | sed 's,[;#][^\042\047]*$,,' > "$2".src || return 1

  for section in $(/var/epages/ini-config.sh -f "$2".src -g) ; do
    /var/epages/ini-config.sh -f "$2".src -s "$section" -g | while read line ; do
      key=$(echo ${line%%=*} | sed -r 's,(^ *| *$),,')
      value=$(echo ${line#*=} | sed -r 's,(^ *| *$),,')
      /var/epages/ini-config.sh -f "$2" -s "$section" -k "$key" -v "$value" -i
    done
  done
  rm -f "$2".src
}

fi

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
