#!/bin/bash

#==============================================================================
# user/group functions
#==============================================================================

fun_account_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export repair_number repair_name

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_httpd_sourced ]] || . $(dirname $BASH_SOURCE)/fun-httpd.sh
export get_httpd_user get_httpd_group get_httpd_groupid

[[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export get_uid get_gid get_group get_grnam
export set_user_info set_group_info get_user_info get_group_info
export add_user add_to_cron_group check_epages_webgroup
export chown_epages chgrp_epages

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
#  get_uid
#..............................................................................
get_uid() {
  local USAGE='
syntax: get_uid USER_NAME
description: returns UID of USER_NAME
'
  local USER_NAME=$1

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $USER_NAME ]] && return 1

  local USER_LINE USER_INFO
  USER_LINE=$(getent passwd "$USER_NAME") || return 1
  USER_INFO=(${USER_LINE//:/ })
  echo ${USER_INFO[2]}
}

#..............................................................................
#  get_gid
#..............................................................................
get_gid() {
  local USAGE='
syntax: get_gid GROUP_NAME
description: returns GID of GROUP_NAME
'
  local GROUP_NAME=$1

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $GROUP_NAME ]] && return 1

  local GROUP_LINE GROUP_INFO
  GROUP_LINE=$(getent group "$GROUP_NAME") || return 1
  GROUP_INFO=(${GROUP_LINE//:/ })
  echo ${GROUP_INFO[2]}
}

#..............................................................................
#  get_group
#..............................................................................
get_group() {
  local USAGE='
syntax: get_group USER_NAME
description: returns primary GID of USER_NAME
'
  local USER_NAME=$1

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $USER_NAME ]] && return 1

  local USER_LINE USER_INFO
  USER_LINE=$(getent passwd "$USER_NAME") || return 1
  USER_INFO=(${USER_LINE//:/ })
  echo ${USER_INFO[3]}
}

#..............................................................................
#  get_grnam
#..............................................................................
get_grnam() {
  local USAGE='
syntax: get_grnam GROUP_ID
description: returns group name of GROUP_ID
'
  local GROUP_ID=$1

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $GROUP_ID ]] && return 1

  local GROUP_LINE GROUP_INFO
  GROUP_LINE=$(getent group "$GROUP_ID") || return 1
  GROUP_INFO=(${GROUP_LINE//:/ })
  echo ${GROUP_INFO[0]}
}

#..............................................................................
# get_user_info
#..............................................................................
get_user_info() {
  local USAGE='
syntax: get_user_info [ENVVAR]
description: find user name (and UID) for ENVVAR from /etc/{default,sysconfig}/epages6
options:
  - ENVVAR: one of: AS AS_RUN WS WS_RUN J MYSQL
returns:
  - USER_INFO is NAME:UID, UID might be empty
examples:
  USER_INFO=$(get_user_info $ENVVAR) || echo ERROR
'
  local ENVVAR=$1

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $ENVVAR ]] && return 1

  local USER_NAME USER_ID
  case $ENVVAR in
    AS) USER_NAME=ep_appl
        USER_ID=$(get_uid $USER_NAME) || USER_ID=$EPAGES_INSTALL_AS_UID ;;
    AS_RUN)
        USER_NAME=$(repair_name ${EPAGES_APPUSER:-$EPAGES_INSTALL_AS_RUNUSER})
        USER_NAME=${USER_NAME:-eprunapp}
        USER_ID=$(get_uid $USER_NAME) || USER_ID=$EPAGES_INSTALL_AS_RUNUID ;;
    WS) USER_NAME=ep_web
        USER_ID=$(get_uid $USER_NAME) || USER_ID=$EPAGES_INSTALL_WS_UID ;;
    WS_RUN)
        USER_NAME=$(repair_name ${EPAGES_WEBUSER:-$EPAGES_INSTALL_WS_RUNUSER})
        USER_NAME=${USER_NAME:-$(get_httpd_user)}
        USER_NAME=${USER_NAME:-eprunweb}
        USER_ID=$(get_uid $USER_NAME) || USER_ID=$EPAGES_INSTALL_WS_RUNUID ;;
    J)
        USER_NAME=$(repair_name ${EPAGESJ_USER:-$EPAGES_INSTALL_EPAGESJ_USER})
        USER_NAME=${USER_NAME:-epagesj}
        USER_ID=$(get_uid $USER_NAME) || USER_ID=$EPAGES_INSTALL_EPAGESJ_UID ;;
    MYSQL) 
        USER_NAME=$(repair_name $EPAGES_INSTALL_MYSQL_UNIX_USER)
        USER_NAME=${USER_NAME:-mysql}
        USER_ID=$(get_uid $USER_NAME) ;;
    *) return 1 ;;
  esac

  echo $USER_NAME:$(repair_number $USER_ID)
  return 0
}

#..............................................................................
# get_group_info
#..............................................................................
get_group_info() {
  local USAGE='
syntax: get_group_info [ENVVAR]
description: find group name (and GID) for ENVVAR from /etc/{default,sysconfig}/epages6
options:
  - ENVVAR: one of: AS WS MYSQL
returns:
  - GROUP_INFO is NAME:GID, GID might be empty
examples:
  GROUP_INFO=$(get_group_info $ENVVAR) || echo ERROR
'
  local ENVVAR=$1

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $ENVVAR ]] && return 1

  local USER_INFO GROUP_NAME GROUP_ID
  case $ENVVAR in
    AS) GROUP_NAME=epages
        GROUP_ID=$(get_gid $GROUP_NAME) || GROUP_ID=$EPAGES_INSTALL_EP_GID ;;
    WS|WS_RUN) USER_INFO=$(get_user_info $ENVVAR)
        if [[ -n $EPAGES_WEBGROUP ]] ; then
          # if EPAGES_WEBGROUP is set, use this one
          GROUP_NAME=${EPAGES_WEBGROUP%%:*}
          GROUP_ID=$(get_gid $GROUP_NAME) || GROUP_ID=${EPAGES_WEBGROUP##*:}
        elif [[ -n $(repair_name $EPAGES_INSTALL_WS_GROUP) ]] ; then
          # if EPAGES_INSTALL_WS_GROUP is set, use this one
          GROUP_NAME=$(repair_name $EPAGES_INSTALL_WS_GROUP)
          GROUP_ID=$(get_gid $GROUP_NAME) || GROUP_ID=$EPAGES_INSTALL_WS_GID
        elif [[ -n $EPAGES_INSTALL_WS_GID ]] ; then
          # if EPAGES_INSTALL_WS_GID is set, use this one
          GROUP_ID=$EPAGES_INSTALL_WS_GID
          GROUP_NAME=$(get_grnam $GROUP_ID) || GROUP_NAME=$(get_httpd_group)
          GROUP_NAME=${GROUP_NAME:-ep_web}
        elif GROUP_ID=$(get_group ${USER_INFO%:*}) ; then
          # if webuser exists, use primary group
          GROUP_NAME=$(get_grnam $GROUP_ID)
        else
          # else use default
          GROUP_NAME=$(get_httpd_group)
          GROUP_ID=$(get_httpd_groupid)
        fi ;;
    MYSQL)
        GROUP_NAME=$(repair_name $EPAGES_INSTALL_MYSQL_UNIX_GROUP)
        USER_INFO=$(get_user_info MYSQL)
        if [[ -n $GROUP_NAME ]] ; then
          # if EPAGES_INSTALL_MYSQL_UNIX_GROUP is set, use this one
          GROUP_ID=$(get_gid $GROUP_NAME)
        elif GROUP_ID=$(get_group ${USER_INFO%:*}) ; then
          # if MySQL user exists, use primary group
          GROUP_NAME=$(get_grnam $GROUP_ID)
        else
          # else use default
          GROUP_NAME=mysql
        fi ;;
    *) return 1 ;;
  esac

  echo $GROUP_NAME:$(repair_number $GROUP_ID)
  return 0
}

#..............................................................................
# set_user_info
#..............................................................................
set_user_info() {
  local USAGE='
syntax: set_user_info [-c] ENVVAR USER_NAME USER_ID
description: writes user information into /etc/{default,sysconfig}/epages6 and
  (with -c) into /srv/epages/etc/created-accounts
options:
  - ENVVAR: one of: AS AS_RUN WS WS_RUN J MYSQL
  - USER_NAME: user name
  - USER_ID: UID
example:
  set_user_info -c "$LOGIN_USER" "$USER_NAME" "$USER_ID"
'
  local NEW_ACCOUNT
  [[ $1 = -c ]] && { NEW_ACCOUNT=1; shift; }
  local ENVVAR=$1
  local USER_NAME=$2
  local USER_ID=$3

  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # edit /srv/epages/etc/created-accounts

  if [[ -n $NEW_ACCOUNT ]] ; then
    EPAGES=${EPAGES:-/srv/epages/eproot}
    local CREATED_ACCOUNTS=$(dirname $EPAGES)/etc/created-accounts
    mkdir -m 755 -p $(dirname $CREATED_ACCOUNTS)
    grep -q "^user $USER_NAME$" $CREATED_ACCOUNTS 2>/dev/null \
    || echo user $USER_NAME >> $CREATED_ACCOUNTS
    chmod 644 $CREATED_ACCOUNTS
  fi

  # edit /etc/sysconfig/epages6

  local VARNAME
  case $ENVVAR in
    AS|WS)         VARNAME=${ENVVAR}_ ;;
    AS_RUN|WS_RUN) VARNAME=$ENVVAR ;;
    J)             VARNAME=EPAGESJ_ ;;
    MYSQL)         VARNAME=MYSQL_UNIX_ ;;
    *) return 1 ;;
  esac

  mkdir -m 755 -p /etc/sysconfig
  /var/epages/ini-config.sh -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_${VARNAME}USER -v $USER_NAME -i
  [[ -n $USER_ID ]] && /var/epages/ini-config.sh -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_${VARNAME}UID -v $USER_ID -i
  chmod 644 /etc/sysconfig/epages6

  # edit /etc/default/epages6 (if exists)

  [[ -f $(etc_default) ]] || return 0
  
  case $ENVVAR in
    AS_RUN) VARNAME=EPAGES_APPUSER ;;
    WS_RUN) VARNAME=EPAGES_WEBUSER ;;
    J)      VARNAME=EPAGESJ_USER ;;
    *) return 0 ;;
  esac

  /var/epages/ini-config.sh -f $(etc_default) -k $VARNAME -v $USER_NAME -i
  return 0
}

#..............................................................................
# set_group_info
#..............................................................................
set_group_info() {
  local USAGE='
syntax: set_group_info [-c] ENVVAR GROUP_NAME GROUP_ID
description: writes group information into /etc/{default,sysconfig}/epages6 and
  (with -c) into /srv/epages/etc/created-accounts
options:
  - ENVVAR: one of: AS WS MYSQL
  - GROUP_NAME: group name
  - GROUP_ID: UID
example:
  set_group_info -c "$LOGIN_GROUP" "$GROUP_NAME" "$GROUP_ID"
'
  local NEW_ACCOUNT
  [[ $1 = -c ]] && { NEW_ACCOUNT=1; shift; }
  local ENVVAR=$1
  local GROUP_NAME=$2
  local GROUP_ID=$3

  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # edit /srv/epages/etc/created-accounts

  if [[ -n $NEW_ACCOUNT ]] ; then
    EPAGES=${EPAGES:-/srv/epages/eproot}
    local CREATED_ACCOUNTS=$(dirname $EPAGES)/etc/created-accounts
    mkdir -m 755 -p $(dirname $CREATED_ACCOUNTS)
    grep -q "^group $GROUP_NAME$" $CREATED_ACCOUNTS 2>/dev/null \
    || echo group $GROUP_NAME >> $CREATED_ACCOUNTS
    chmod 644 $CREATED_ACCOUNTS
  fi

  # edit /etc/sysconfig/epages6

  local VARNAME
  case $ENVVAR in
    AS) VARNAME=EP ;;
    WS) VARNAME=WS ;;
    MYSQL) VARNAME=MYSQL_UNIX ;;
    *) return 1 ;;
  esac

  mkdir -m 755 -p /etc/sysconfig
  /var/epages/ini-config.sh -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_${VARNAME}_GROUP -v $GROUP_NAME -i
  [[ -n $GROUP_ID ]] && /var/epages/ini-config.sh -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_${VARNAME}_GID -v $GROUP_ID -i
  chmod 644 /etc/sysconfig/epages6

  # edit /etc/default/epages6 (if exists)

  [[ $ENVVAR != WS || -z $GROUP_ID ]] && return 0
  [[ -f $(etc_default) ]] || return 0

  /var/epages/ini-config.sh -f $(etc_default) -k EPAGES_WEBGROUP -v "$GROUP_NAME:$GROUP_ID" -i
  return 0
}

#..............................................................................
# add_user
#..............................................................................
add_user() {
  local USAGE='
syntax: add_user -u ENVVAR [-g ENVVAR] [-G ENVVAR,ENVVAR,...] [-s SHELL]
description: creates/changes user/group
  - If -g is used, function does nothing if user already exists
  - If -g is not used but -G, function does nothing unless user exists
options:
  - ENVVAR: one of: AS AS_RUN WS WS_RUN J MYSQL
  - SHELL: login shell in (/usr)?/s?bin like bash (default) or nologin
examples:
  add_user -u WS_RUN -g WS -s nologin
  add_user -u J -g AS
  add_user -u AS_RUN -g AS -G WS
  add_user -u WS -g WS
  add_user -u AS -g AS -G WS
  add_user -u AS -G MYSQL
  add_user -u AS_RUN -G MYSQL
  add_user -u MYSQL -G AS
'
  local OPTIND=0
  local LOGIN_USER LOGIN_GROUP OTHER_GROUPS LOGIN_SHELL
  while getopts u:g:G:s: c $@ ; do
    case $c in
      u) LOGIN_USER=$OPTARG ;;
      g) LOGIN_GROUP=$OPTARG ;;
      G) OTHER_GROUPS=$OPTARG ;;
      s) LOGIN_SHELL=$OPTARG ;;
      *) echo "$USAGE" 1>&2; return 1 ;;
    esac
  done
  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z $LOGIN_USER ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -z "$LOGIN_GROUP$OTHER_GROUPS" ]] && { echo -e "ERROR: neither -g nor -G defined\n$USAGE" 1>&2; return 1; }

  local USER_NAME USER_ID
  USER_NAME=$(get_user_info $LOGIN_USER) || { echo -e "ERROR: unknown user: $LOGIN_USER\n$USAGE" 1>&2; return 1; }
  [[ $USER_NAME = ${USER_NAME#*:} ]] || { USER_ID=${USER_NAME##*:}; USER_NAME=${USER_NAME%:*}; }

  # add groups
  local GROUP_NAME GROUP_ID gGROUP GGROUPS
  for i in $LOGIN_GROUP $OTHER_GROUPS ; do
    unset GROUP_NAME GROUP_ID SET_GROUP
    GROUP_NAME=$(get_group_info $i) || { echo -e "ERROR: unknown group: $i\n$USAGE" 1>&2; return 1; }
    [[ $GROUP_NAME = ${GROUP_NAME#*:} ]] || { GROUP_ID=${GROUP_NAME##*:}; GROUP_NAME=${GROUP_NAME%:*}; }
    [[ -n $LOGIN_GROUP && -z $gGROUP ]] && gGROUP=$GROUP_NAME || GGROUPS="$GGROUPS,$GROUP_NAME"

    if grep -q "^$GROUP_NAME:" /etc/group ; then
      SET_GROUP=1
    else
      /usr/sbin/groupadd ${GROUP_ID:+-o -g $GROUP_ID} $GROUP_NAME && SET_GROUP=-c
    fi
    [[ -n $SET_GROUP ]] && set_group_info ${SET_GROUP#1} "$i" "$GROUP_NAME" "$GROUP_ID"
  done

  gGROUP=${gGROUP:+-g $gGROUP}
  GGROUPS=${GGROUPS:+-G ${GGROUPS#,}}
  local USER_EXISTS SET_USER
  getent passwd "$USER_NAME" >/dev/null && USER_EXISTS=1 || USER_EXISTS=

  # add user
  if [[ -z $USER_EXISTS && -n $gGROUP ]] ; then
    if [[ -n $LOGIN_SHELL ]] ; then
      local FOUND
      for i in /bin /usr/bin /sbin /usr/sbin; do
        [[ -x $i/$LOGIN_SHELL ]] && { FOUND=$i ; break; }
      done
      [[ -n $FOUND ]] && LOGIN_SHELL="-s $FOUND/$LOGIN_SHELL" || LOGIN_SHELL=
    fi
    /usr/sbin/useradd -m ${USER_ID:+-o -u $USER_ID} $gGROUP $GGROUPS $LOGIN_SHELL $USER_NAME && SET_USER=-c
  fi

  # modify user
  if [[ -n $USER_EXISTS ]] ; then
    SET_USER=1
    if [[ -n $GGROUPS ]] ; then
      # SuSE uses 'usermod -A groups', other use 'usermod -a -G groups'
      [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%s} ]] && GGROUPS="-a $GGROUPS" || GGROUPS=${GGROUPS//-G/-A}
      /usr/sbin/usermod $GGROUPS $USER_NAME
    fi
  fi

  [[ -n $SET_USER ]] && set_user_info ${SET_USER#1} "$LOGIN_USER" "$USER_NAME" "$USER_ID"
}

#..............................................................................
# add_to_cron_group
#..............................................................................
add_to_cron_group() {
  local USAGE="
syntax: add_to_cron_group USER...
description: adds USER(s) to group which runs cron. we need this because apparmor
  secured linux doesn't allow unprivileged users to use crontab (see EPG-9706).
example: add_to_cron_group AS WS AS_RUN
"
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local CRON_TAB CRON_GROUP USER_NAME ADD
  CRON_TAB=$(type -p crontab) || return
  [[ -x $CRON_TAB ]] || return
  CRON_GROUP=$(find "$CRON_TAB" -perm -g=rx -printf "%g") || return

  # SuSE uses 'usermod -A groups', other use 'usermod -a -G groups'
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%s} ]] && ADD="-a -G" || ADD=-A

  # debian requires some changes
  if [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%d} ]] ; then
    chmod g+s $CRON_TAB
    install -g root -o root -m 755 -d /var/spool/cron
    install -g crontab -o root -m 1730 -d /var/spool/cron/crontabs
  fi
  for i in $@ ; do
    USER_NAME=$(get_user_info $i) || continue
    /usr/sbin/usermod $ADD $CRON_GROUP ${USER_NAME%:*}
    # debian requires some changes
    if [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%d} ]] ; then
      touch /var/spool/cron/crontabs/${USER_NAME%:*}
      chown ${USER_NAME%:*}:crontab /var/spool/cron/crontabs/${USER_NAME%:*}
      chmod 600 /var/spool/cron/crontabs/${USER_NAME%:*}
    fi
  done
}

#..............................................................................
# check_epages_webgroup
#..............................................................................
check_epages_webgroup() {
  local USAGE='
syntax: check_epages_webgroup {*start*/*stop*}
description: check if environment variable EPAGES_WEBGROUP is set correctly
options:
  *start*: add missing UNIX user as defined in EPAGES_WEBGROUP=
returns: GID of epages web group
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local GROUP=(${EPAGES_WEBGROUP//:/ })

  if [[ ${#GROUP[@]} = 0 ]] ; then
    GROUP[0]=ep_web
    GROUP[1]=$(get_gid ${GROUP[0]}) || {
      echo "EPAGES_WEBGROUP=: no GID found for ep_web" 1>&2
      return 1
    }
  elif [[ ${#GROUP[@]} = 1 ]] ; then
    echo "EPAGES_WEBGROUP=$EPAGES_WEBGROUP: wrong format of environment variable" 1>&2
    return 1
  elif [[ -z ${GROUP[1]} || -n ${GROUP[1]//[0-9]/} ]] ; then
    echo "EPAGES_WEBGROUP=$EPAGES_WEBGROUP: '${GROUP[1]}' not a GID" 1>&2
    return 1
  fi

  case $1 in
    start|*start_httpd|start_ssl|start_service) : ;;
    *) echo ${GROUP[1]} ; return 0 ;;
  esac

  # if *start*, check and add EPAGES_WEBGROUP (if necessary)

  local GROUP_ID=$(get_gid ${GROUP[0]})
  if [[ $GROUP_ID != ${GROUP[1]} ]] ; then
    echo "EPAGES_WEBGROUP=$EPAGES_WEBGROUP: void group name:id pair" 1>&2
    return 1
  elif [[ -z $GROUP_ID ]] && ! /usr/sbin/groupadd -g ${GROUP[1]} ${GROUP[0]} ; then
    echo "EPAGES_WEBGROUP=$EPAGES_WEBGROUP: command failed: groupadd -g ${GROUP[1]} ${GROUP[0]}" 1>&2
    return 1
  fi

  # SuSE uses 'usermod -A groups', other use 'usermod -a -G groups'
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%s} ]] && ADD="-a -G" || ADD=-A

  local USER_NAME
  for i in AS AS_RUN ; do
    USER_NAME=$(get_user_info $i) || continue
    /usr/sbin/usermod $ADD ${GROUP[0]} ${USER_NAME%:*} || {
      echo "EPAGES_WEBGROUP=$EPAGES_WEBGROUP: adding user '${USER_NAME%:*}' to group '${GROUP[0]}' failed"
      return 1
    }
  done

  echo ${GROUP[1]}
  return 0
}

#..............................................................................
# chgrp_epages
#..............................................................................
chgrp_epages() {
  local USAGE='
syntax: chgrp_epages [-R] ENVVAR FILES
description: chgrp [-R] FILES with $(get_group_info ENVVAR)
ENVVAR: one of: AS WS MYSQL
'
  local R
  [[ $1 = -R ]] && { R=-R; shift; }
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local ENVVAR=$1 ; shift
  local group=$(get_group_info $ENVVAR) || return 1
  chgrp -fL $R ${group%%:*} $@

  if [[ $ENVVAR = WS ]] ; then
    local user=$(get_user_info WS_RUN)
    chown -fLR ${user%%:*} "$HTTPD_ROOT"/cache 2>/dev/null
  fi
}

#..............................................................................
# chown_epages
#..............................................................................
chown_epages() {
  local USAGE='
syntax: chown_epages [-R] ENVVAR FILES
description: chown [-R] FILES with $(get_user_info ENVVAR)
ENVVAR: one of: AS AS_RUN WS WS_RUN J MYSQL
'
  local R
  [[ $1 = -R ]] && { R=-R; shift; }
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local ENVVAR=$1 ; shift
  local user=$(get_user_info $ENVVAR) || return 1
  chown -fL $R ${user%%:*} $@
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
