#!/bin/bash

#==============================================================================
# host name and IP address
#==============================================================================

fun_host_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export guess_addr_type check_IPv4 check_IPv6 check_hostname normalize_IPv6_address
export my_host my_ips get_host_info get_ip_info resolve_host_info set_etc_hosts
export get_host_name get_host_ip

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# my_host
#..............................................................................
my_host() {
  local USAGE='
syntax: my_host IP/hostname
description: checks if IP/hostname belongs to host that executes my_host
'
  [[ -z $1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  local ADDR=$(eval echo $1)
  local MY_HOST=$(hostname --fqdn)

  [[ 0.0.0.0 = $ADDR || $MY_HOST = $ADDR || ${MY_HOST%%.*} = $ADDR ]] && return 0

  local IP
  for IP in $(PATH=/sbin:/usr/sbin:$PATH ip addr show \
              | awk '/^[\t ]*inet/{sub(/\/.*/,"",$2); print $2}') ; do
    [[ $IP = $ADDR ]] && return 0
    getent hosts $ADDR | grep -Pq "^$IP\s" && return 0
    dig $ADDR +time=1 +short +search +answer | grep -q "^$IP$" && return 0
  done
  return 1
}

#..............................................................................
# my_ips
#..............................................................................
my_ips() {
  local USAGE='
syntax: my_ips
description: echoes IPs from $(ip addr show scope global)
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  PATH=/sbin:/usr/sbin:$PATH ip addr show scope global \
  | awk '/^[\t ]*inet/{sub(/\/.*/,"",$2); print $2}'
}

#..............................................................................
# guess_addr_type
#..............................................................................
guess_addr_type() {
  local USAGE='
syntax: guess_addr_type ADDR
description: echoes type of ADDR: IPv4, IPv6, hostname or nothing (error)
examples:
  [[ $(guess_addr_type epages.com) = hostname ]] && check_hostname epages.com
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local ADDR=$1

  # return if ADDR is empty
  [[ -z $ADDR ]] &&  return
  # return if ADDR contains void characters
  echo $ADDR | tr -d '[.:0-9a-z\300-\377-]' | grep -q . && return
  # return if ADDR contains '..'
  [[ ${ADDR/../} != $ADDR ]] && return
  # return if ADDR starts/ends with '.'
  [[ ${ADDR#.} != $ADDR || ${ADDR%.} != $ADDR ]] && return

  [[ -z ${ADDR//[.:0-9a-f]/} && $ADDR != ${ADDR/:/} ]] && { echo IPv6 ; return; }
  [[ -z ${ADDR//[0-9]/} ]] && { echo hostname ; return; }
  [[ -z ${ADDR//[.0-9]/} ]] && { echo IPv4 ; return; }
  echo hostname ; return
}

#..............................................................................
# normalize_IPv6_address
#..............................................................................
normalize_IPv6_address () {
  local USAGE='
syntax: normalize_IPv6_address IP
description: echoes normalized IP
examples:
  IP=$(normalize_IPv6_address ::0)
'

  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# -eq 0 ]] && return

  local IP=$1
  local PRE=$(echo ${IP%::*})
  local POST=$(echo ${IP#*::})
  local PART_MAX=8
  [[ ${IP/./} = $IP ]] || PART_MAX=7

  if [[ $PRE != $IP ]] ; then
    local PRE_PARTS=0
    [[ -n $PRE ]] && { PRE_PARTS=$(echo "$PRE" | tr ':' '\n' | wc -l); PRE="$PRE:"; }
    local POST_PARTS=0
    [[ -n $POST ]] && { POST_PARTS=$(echo "$POST" | tr ':' '\n' | wc -l); POST=":$POST"; }
    local IN_PARTS=$(($PART_MAX - $PRE_PARTS - $POST_PARTS))
    local NULL=0
    for i in $(seq 2 $IN_PARTS) ; do NULL="$NULL:0" ; done
    IP="$PRE$NULL$POST"
  fi

  echo "$IP" | sed -r 's,:0+([^:]),:\1,g;s,^0+([^:]),\1,' | tr A-Z a-z
}

#..............................................................................
# check_hostname
#..............................................................................
check_hostname() {
  local USAGE='
syntax: check_hostname HOST_NAME
description: returns 0 - correct hostname format, 1 - else
examples:
  check_hostname epages.com && echo epages.com is hostname
'

  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 5; }

  local HOST_NAME=$1

  # no hostname
  [[ $(guess_addr_type $HOST_NAME) = hostname ]] || return 1
  # contains ':'
  [[ ${ADDR/:/} != $ADDR ]] && return 1
  # is longer than 253 chars
  [[ ${#ADDR} -gt 253 ]] && return 1

  type -t idn >/dev/null && HAS_IDN=1 || unset HAS_IDN
  for i in ${ADDR//./ } ; do
    # starts/ends with [-]
    [[ $i != ${i#-} || $i != ${i%-} ]] && return 1
    # encode punycode if hostname is IDN (it contains [\300-\377])
    if [ -n "$HAS_IDN" -a -n "${i//[0-9a-z-]/}" ] ; then
      i="xn--$(LANG=en_US idn --quiet -e "$i")"
    fi
    # part is longer than 63 chars
    [ ${#i} -gt 63 ] && return 1
  done

  return 0
}

#..............................................................................
# check_IPv4
#..............................................................................
check_IPv4() {
  local USAGE='
syntax: check_IPv4 IP
description: returns 0 - correct IPv4 format, 1 - else
examples:
  check_IPv4 1.2.3.4 && echo 1.2.3.4 is IPv4
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 5; }

  local IP=$1

  # no IPv4
  [[ $(guess_addr_type $IP) = IPv4 ]] || return 1

  # doesn't contain exactly 3 '.'
  local NO_DOT=${IP//./}
  [[ $((${#IP} - ${#NO_DOT})) = 3 ]] || return 1

  for i in ${IP//./ } ; do
    # part starts with 0
    [[ ${#i} -gt 1 && ${i#0} != $i ]] && return 1
    # part is > 255
    [[ ${#i} -gt 3 || $i -gt 255 ]] && return 1
  done
  return 0
}

#..............................................................................
# check_IPv6
#..............................................................................
check_IPv6() {
  local USAGE='
syntax: check_IPv6 IP
description: returns 0 - correct IPv6 format, 1 - else
examples:
  check_IPv6 ::ffff:1.2.3.4 && echo ::ffff:1.2.3.4 is IPv6
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 5; }

  local IP=$1

  # no IPv6
  [[ $(guess_addr_type $IP) = IPv6 ]] || return 1

  set -- $(echo ${IP//:/ })
  local IPv4=${@:$#} # last part

  if [[ ${IPv4/./} != $IPv4 ]] ; then
    # addresses like ::ffff:172.20.50.1
    check_IPv4 $IPv4 || return 1
    IP=$(normalize_IPv6_address "${IP%$IPv4}")
    # IPv6 part must be :: or ::ffff:
    [[ -z $IP || $IP = 0:0:0:0:0:0:0:0 || $IP = 0:0:0:0:0:0:ffff: ]] || return 1
    return 0
  fi

  # contains less than 2 ':'
  local NO_COLON=${IP//:/}
  [[ $((${#IP} - ${#NO_COLON})) -lt 2 ]] && return 1
  # contains more than 8 ':'
  [[ $((${#IP} - ${#NO_COLON})) -gt 8 ]] && return 1
  # doesn't contains exactly 7 ':'
  [[ ${IP/::/} = $IP && $((${#IP} - ${#NO_COLON})) -ne 7 ]] && return 1
  # contains ':::'
  [[ ${IP/:::/} != $IP ]] && return 1
  # contains ::.*::
  [[ ${IP/::/} != ${IP//::/} ]] && return 1
  # starts with ':' (but not with '::')
  [[ ${IP#:} != $IP && ${IP#::} = $IP ]] && return 1
  # ends with ':' (but not with '::')
  [[ ${IP%:} != $IP && ${IP%::} = $IP ]] && return 1

  for i in ${IP//:/ } ; do
    # part is > ffff
    [[ ${#i} -gt 4 ]] && return 1
  done
  return 0
}

#..............................................................................
# get_host_info
#..............................................................................
get_host_info() {
  local USAGE='
syntax: get_host_info SERVER
description: echoes hostname/IP for SERVER as defined in
   /etc/{sysconfig,default}/epages6 (or nothing unless valid defined)
options:
  - SERVER: one of: AS RR WS J EPAGESJ MYSQL
returns:
  - 0 - hostname/IP is valid
  - 1 - hostname/IP is empty
  - 4 - hostname/IP is void
  - 5 - other error
examples:
  WS_HOST=$(get_host_info WS) ; [[ $? = 4 ]] && echo hostname/IP invalid
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 5; }

  [[ -f $(readlink -f $(etc_default)) ]] && . $(etc_default)
  [[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6

  local SERVER=$1
  [[ $SERVER =  J ]] && SERVER=EPAGESJ
  local HOST_OR_IP=$(eval echo '$'EPAGES_INSTALL_${SERVER}_HOST)
  HOST_OR_IP=${HOST_OR_IP%:ip6} # legacy

  if [[ -z $HOST_OR_IP ]] ; then
    case $SERVER in
      EPAGESJ) [[ -n $EPAGESJ_HOST ]] && HOST_OR_IP=$EPAGESJ_HOST ;;
      MYSQL) [[ -n $EPAGES_MYSQL_CONNECTION &&
                $EPAGES_MYSQL_CONNECTION = ${EPAGES_MYSQL_CONNECTION%/*} ]] \
             && HOST_OR_IP=${EPAGES_MYSQL_CONNECTION%%;*} ;;
    esac
    HOST_OR_IP=${HOST_OR_IP//[][]/}
  fi

  [[ -z $HOST_OR_IP ]] && return 1

  local RESULT=0
  case $(guess_addr_type "$HOST_OR_IP") in
    hostname) check_hostname "$HOST_OR_IP" || RESULT=4 ;;
    IPv4) check_IPv4 "$HOST_OR_IP" || RESULT=4 ;;
    IPv6) check_IPv6 "$HOST_OR_IP" || RESULT=4 ;;
    *) RESULT=4 ;;
  esac

  [[ $RESULT = 0 ]] && echo "$HOST_OR_IP"
  return $RESULT
}

#..............................................................................
# get_ip_info
#..............................................................................
get_ip_info() {
  local USAGE='
syntax: get_ip_info SERVER
description: echoes *_HOSTIP for SERVER as defined in
   /etc/sysconfig/epages6 (or nothing unless valid defined)
options:
  - SERVER: one of: AS RR WS J EPAGESJ MYSQL
returns:
  - 0 - IP is valid
  - 1 - IP is empty
  - 4 - IP is void
  - 5 - other error
examples:
  WS_IP=$(get_ip_info WS) ; [[ $? = 4 ]] && echo IP invalid
'
  [[ $# -ne 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 5; }

  [[ -f $(readlink -f $(etc_default)) ]] && . $(etc_default)
  [[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6

  local SERVER=$1
  [[ $SERVER =  J ]] && SERVER=EPAGESJ
  local HOSTIP=$(eval echo '$'EPAGES_INSTALL_${SERVER}_HOSTIP)

  [[ -z $HOSTIP ]] && return 1

  local RESULT=0
  case $(guess_addr_type "$HOSTIP") in
    IPv4) check_IPv4 "$HOSTIP" || RESULT=4 ;;
    IPv6) check_IPv6 "$HOSTIP" || RESULT=4 ;;
    *) RESULT=4 ;;
  esac

  [[ $RESULT = 0 ]] && echo "$HOSTIP"
  return $RESULT
}

#..............................................................................
# get_ip_by_name
#..............................................................................
get_ip_by_name() {
  local USAGE='
syntax: get_ip_by_name HOST_NAME
description: echoes IP(s) for HOST_NAME
examples:
  IP_LIST=$(get_ip_by_name google.com)
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# = 0 ]] && return

  local HOST_NAME=$1
  [[ $(guess_addr_type "$HOST_NAME") = hostname ]] || return
  
  local HOST_IP=$(( dig $HOST_NAME +time=1 +short +search +answer
                    dig $HOST_NAME -t AAAA +time=1 +short +search +answer
                    getent hosts $HOST_NAME | awk '{print $1}'
                  ) | tr ' ' '\n' | sort | uniq | tr '\n' ' ')
  local ADDR GLOBAL_IP
  for ADDR in $(my_ips) ; do
    [[ ' '$HOST_IP' ' == *' '$ADDR' '* ]] && GLOBAL_IP="$GLOBAL_IP $ADDR"
  done
  # prefer IPs with global scope
  echo ${GLOBAL_IP:-$HOST_IP}
}

#..............................................................................
# get_name_by_ip
#..............................................................................
get_name_by_ip() {
  local USAGE='
syntax: get_name_by_ip IP
description: echoes hostname(s) for IP
examples:
  HOST_NAME_LIST=$(get_name_by_ip 173.194.69.100)
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# = 0 ]] && return

  local IP=$1
  local ADDR_TYPE=$(guess_addr_type "$IP")
  [[ $ADDR_TYPE = ${ADDR_TYPE#IP} ]] && return

  ( dig -x $IP +time=1 +short +search +answer | sed 's,\.$,,'
    dig -x $IP -t AAAA +time=1 +short +search +answer | sed 's,\.$,,'
    getent hosts $IP | sed -r 's,^\s*\S+\s*,,'
  ) | tr ' ' '\n' | sort | uniq | tr '\n' ' '
}

#..............................................................................
# get_host_name
#..............................................................................
get_host_name() {
  local USAGE='
syntax: get_host_name [SERVER]
description: hostname of SERVER (or $(hostname) or localhost)
options:
  - SERVER: one of: AS RR WS J MYSQL
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SERVER=$1
  local HOST_OR_IP

  if [[ -n $SERVER ]] ; then
    HOST_OR_IP=$(get_host_info $SERVER)
    [[ $(guess_addr_type "$HOST_OR_IP") = hostname ]] && { echo $HOST_OR_IP; return; }

    HOST_OR_IP=${HOST_OR_IP:-$(get_ip_info $SERVER)}
    if [[ -n $HOST_OR_IP ]] ; then
      HOST_OR_IP=$(get_name_by_ip $HOSTIP)
      HOST_OR_IP=${HOST_OR_IP%% *}
    fi
  fi

  HOST_OR_IP=${HOST_OR_IP:-$(hostname --fqdn)}
  echo ${HOST_OR_IP:-localhost}
}

#..............................................................................
# get_host_ip
#..............................................................................
get_host_ip() {
  local USAGE='
syntax: get_host_ip [SERVER]
description: IP of SERVER (or $(host $(hostname)) or 127.0.0.1
options:
  - SERVER: one of: AS RR WS J MYSQL
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SERVER=$1
  local HOST_OR_IP ADDR_TYPE

  if [[ -n $SERVER ]] ; then
    HOST_OR_IP=$(get_ip_info $SERVER)
    [[ -n $HOST_OR_IP ]] && { echo $HOST_OR_IP; return; }

    HOST_OR_IP=$(get_host_info $SERVER)
    ADDR_TYPE=$(guess_addr_type "$HOST_OR_IP")
    [[ $ADDR_TYPE = ${ADDR_TYPE#IP} ]] || { echo $HOST_OR_IP; return; }
  fi

  HOST_OR_IP=$(get_ip_by_name ${HOST_OR_IP:-$(hostname)})
  HOST_OR_IP=${HOST_OR_IP%% *}

  echo ${HOST_OR_IP:-127.0.0.1}
}

#..............................................................................
# resolve_host_info
#..............................................................................
resolve_host_info() {
  local USAGE='
syntax: resolve_host_info [SERVER]
description: echoes IPs/hostnames for SERVER (myself if undef)
options:
  - SERVER: one of: AS RR WS J MYSQL
examples:
  ADDR_LIST=$(resolve_host_info WS)
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SERVER=$1
  local HOSTIP HOST_OR_IP

  if [[ -n $SERVER ]] ; then
    HOSTIP=$(get_ip_info $SERVER)
    HOST_OR_IP=$(get_host_info $SERVER)
    [[ $HOSTIP = $HOST_OR_IP ]] && HOSTIP=
  fi
  local ADDR="$HOST_OR_IP $HOSTIP"
  HOST_OR_IP=${HOST_OR_IP:-$(get_host_name $SERVER)}
  ADDR="$ADDR $HOST_OR_IP"

  case $(guess_addr_type "$HOST_OR_IP") in
    IP*) ADDR="$ADDR $(get_name_by_ip $HOSTIP) $(get_name_by_ip $HOST_OR_IP)" ;;
    *) ADDR="$ADDR ${HOST_OR_IP%%.*} $(get_name_by_ip $HOSTIP) $(get_ip_by_name $HOST_OR_IP)" ;;
  esac

  local UNIQ_ADDR
  for HOST_OR_IP in $ADDR ; do
    [[ ' '$UNIQ_ADDR' ' == *' '$HOST_OR_IP' '* ]] || UNIQ_ADDR="$UNIQ_ADDR $HOST_OR_IP"
  done
  echo ${UNIQ_ADDR# }
}

#..............................................................................
# set_etc_hosts
#..............................................................................
set_etc_hosts () {
  local USAGE='
syntax: set_etc_hosts SERVER...
description: changes /etc/hosts according to DNS (if not equal)
  drops duplicate entries and 127. entries with hostname != localhost
options:
  - SERVER: one of: AS RR WS J MYSQL
examples:
  set_etc_hosts WS
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# = 0 ]] && return

  local SERVER
  local CHANGES=$(for SERVER in $@ ; do
    local HOSTIP=$(get_ip_info $SERVER)
    local HOST_OR_IP=$(get_host_info $SERVER)

    # do nothing if IP is explicitely set
    [[ -n $HOSTIP ]] && continue
    local ADDR_TYPE=$(guess_addr_type "$HOST_OR_IP")
    [[ $ADDR_TYPE != ${ADDR_TYPE#IP} ]] && continue

    # do nothing if hostname is unknown
    HOST_OR_IP=${HOST_OR_IP:-$(hostname)}
    [[ -z $HOST_OR_IP ]] && continue
  
    local IP_BY_DNS=$(dig $HOST_OR_IP +time=1 +short +search +answer | head -1)
    local IP_BY_FILE=$(getent hosts $HOST_OR_IP | head -1)
    local HOSTNAME_BY_FILE=$(echo "$IP_BY_FILE" | sed -r 's,^\s*\S+\s*,,')
    IP_BY_FILE=$(echo "$IP_BY_FILE" | awk '{print $1}')

    # do nothing if $IP_BY_DNS = $IP_BY_FILE
    [[ $IP_BY_DNS = $IP_BY_FILE ]] && continue
    # do nothing if $IP_BY_DNS is empty
    [[ -z $IP_BY_DNS ]] && continue
    echo "$IP_BY_DNS $HOSTNAME_BY_FILE"
  done | sort | uniq)

  [[ -z $CHANGES ]] && return

  backup_file /etc/hosts
  cat /etc/hosts | while read ip hostnames ; do
    # drop duplicate entries
    echo "$CHANGES" | grep -q "^$ip " && continue
    # drop 127. entries with hostname != localhost
    if [[ $ip != ${ip#127.} ]] ; then
      echo $hostnames | tr ' ' '\n' | egrep -v '(localhost|loopback)' | grep -q . && continue
    fi
    echo "$ip $hostnames"
  done > /etc/hosts.epages.tmp
  echo "$CHANGES" > /etc/hosts
  cat /etc/hosts.epages.tmp >> /etc/hosts
  rm -f /etc/hosts.epages.tmp
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
