#!/bin/bash

#==============================================================================
# epages functions
#==============================================================================

fun_epages_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export extract_host_from_url extract_port_from_url chkconfig_do

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export get_host_name

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_java_sourced ]] || . $(dirname $BASH_SOURCE)/fun-java.sh
export mongod_name test_version

[[ -n $fun_pkg_sourced ]] || . $(dirname $BASH_SOURCE)/fun-pkg.sh
export pkg_installed

[[ -n $config_sourced ]] || . $(dirname $BASH_SOURCE)/config.sh
export EPAGESJ_DROP_CORES EPAGESJ_DROP_CORES_FOR_VERSION

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export epagesj_rpmsave_value write_epagesj_value start_epagesj update_mongod_conf
export install_epagesj_conf update_epagesj_conf update_epagesj_conf_xml
export findCommandLineArguments epagesj_drop_cores

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# start_epagesj
#..............................................................................
start_epagesj () {
  local USAGE='
syntax: start_epagesj [epagesj epages-solr epages-solr-slave]
arguments: if one/more of arguments are given only those services are started
description: adds and starts epagesj/solr services
'
  local -A START_SERVICE
  if [[ $# = 0 ]] ; then
    START_SERVICE[epagesj]=1
    START_SERVICE[epages-solr]=1
    START_SERVICE[epages-solr-slave]=1
  else
    while [[ $# != 0 ]] ; do
      case $1 in
       epagesj) START_SERVICE[epagesj]=1 ; shift ;;
       epages-solr) START_SERVICE[epages-solr]=1 ; shift ;;
       epages-solr-slave) START_SERVICE[epages-solr-slave]=1 ; shift ;;
       *) echo "$USAGE" 1>&2; return 1 ;;
      esac
    done
  fi

  local INIT_DIR="${RPM_INSTALL_PREFIX:-/srv/epages}"/etc/init.d

  chmod 755 "$INIT_DIR" "${INIT_DIR%/*}"
  # here, order is important: you can only add a service (like epages-solr)
  # if Required-Start: services (like epagesj) were added before
  if type -p systemctl >/dev/null ; then systemctl daemon-reload ; fi
  for i in epagesj epages-solr epages-solr-slave ; do
    chmod 740 "$INIT_DIR"/$i
    chkconfig_do --del $i >/dev/null 2>&1
    [[ -f "$(readlink -qf /etc/init.d/$i)" ]] || ( cd /etc/init.d ; ln -fs "$INIT_DIR"/$i )
    chkconfig_do --add $i >/dev/null 2>&1
    chown root:epages "$INIT_DIR"/$i
    [[ -n ${START_SERVICE[$i]} ]] || continue
    /etc/init.d/$i status >/dev/null && DO=restart || DO=start
    /etc/init.d/$i $DO
  done
}

#..............................................................................
# epagesj_rpmsave_value
#..............................................................................
epagesj_rpmsave_value () {
  local USAGE='
syntax: epagesj_rpmsave_value SECTION KEY
description: get value of KEY in SECTION of epagesj.conf.rpmsave
'
  [[ $# -ne 2 ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILE="$EPAGESJ_CONFIG"/epagesj.conf.rpmsave
  [[ -f $FILE ]] || { echo undef; return; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  $INI_CONFIG -f "$FILE" -s "$1" -G | grep -qP "^\s*$2\s*=" || {
    echo undef; return
  }
  $INI_CONFIG -f "$FILE" -s "$1" -k "$2" -G
}

#..............................................................................
# write_epagesj_value
#..............................................................................
write_epagesj_value () {
  local USAGE='
syntax: write_epagesj_value SECTION KEY VALUE
description: sets VALUE of KEY in SECTION of epagesj.conf
'
  [[ $# -lt 3 ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILE="$EPAGESJ_CONFIG"/epagesj.conf

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  $INI_CONFIG -f "$FILE" -s "$1" -k "$2" -v "\"$3\"" -i
}

#..............................................................................
# update_mongod_conf
#..............................................................................
update_mongod_conf () {
  local USAGE='
syntax: update_mongod_conf
description: updates /etc/mongod.conf as defined in /etc/sysconfig/epages6
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})

  [[ -f $(readlink -f "$ETC_DEFAULT") ]] && . "$ETC_DEFAULT"

  local CONF=${MONGOD_CONF:-/etc/$(mongod_name).conf}
  [[ -f $CONF ]] || return

  local used_port=$($INI_CONFIG -f "$CONF" -k port -G)
  if [[ ${EPAGES_INSTALL_MONGO_PORT:-27017} != ${used_port:-27017} ]] ; then
     $INI_CONFIG -f "$CONF" -k port -v "$EPAGES_INSTALL_MONGO_PORT" -i
  fi

  # if bind_ip is set and differs from MONGO_HOST, unset it
  local bind_ip=$($INI_CONFIG -f "$CONF" -k bind_ip -G)
  if [[ -n $bind_ip && $EPAGES_INSTALL_MONGO_HOST != $bind_ip ]] ; then
     sed -ri 's,^(\s*bind_ip\s*=.*),#\1,' "$CONF"
  fi

  smallfiles=$($INI_CONFIG -f "$CONF" -k smallfiles -G)
  [[ -z $smallfiles ]] && $INI_CONFIG -f "$CONF" -k smallfiles -v true -i

  # if EPAGES_ENABLE_IPv6 is set then define key ipv6=true in /etc/mongod.conf
  [[ $EPAGES_ENABLE_IPv6 = 1 ]] && $INI_CONFIG -f "$CONF" -k ipv6 -v true -i
}

#..............................................................................
# install_epagesj_conf
#..............................................................................
install_epagesj_conf() {
  local USAGE='
syntax: install_epagesj_conf
description: equalizes variables between $EPAGESJ_CONFIG/epagesj.conf
  and /etc/sysconfig/epages6
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  local ETC_INSTALL=/etc/sysconfig/epages6

  [[ -f $(readlink -f "$ETC_DEFAULT") ]] && . "$ETC_DEFAULT"
  [[ -f $ETC_INSTALL ]] && . $ETC_INSTALL

  local CONF="$EPAGESJ_CONFIG"/epagesj.conf
  [[ -f "$CONF" ]] || return

  # [Java]

  local EPAGESJ_OPTS=${EPAGESJ_SERVER_OPTS:-$EPAGES_INSTALL_EPAGESJ_OPTS}
  local JAVA_OPTS=$($INI_CONFIG -f "$CONF" -s Java -k commandLineArguments -G)
  write_epagesj_value Java commandLineArguments "$JAVA_OPTS${EPAGESJ_OPTS:+ $EPAGESJ_OPTS}"

  local host port socket
  local defaultport=(8088 8983 8985 27017)
  local envvar=(EPAGESJ SOLR SOLRSLAVE MONGO)
  local defaulthost=$(get_host_name J)

  for i in 0 1 2 3 ; do
    host[$i]=$(eval echo '$'EPAGES_INSTALL_${envvar[$i]}_HOST)
    host[$i]=${host[$i]%%:ip6}
    host[$i]=${host[$i]:-$defaulthost}

    port[$i]=$(eval echo '$'EPAGES_INSTALL_${envvar[$i]}_PORT)
    port[$i]=$(repair_number "${port[$i]}")
    port[$i]=${port[$i]:-${defaultport[$i]}}

    if [[ ${host[$i]} = ${host[$i]#*:} ]] ; then
      socket[$i]="${host[$i]}:${port[$i]}"
    else # IPv6
      socket[$i]="[${host[$i]}]:${port[$i]}"
    fi
  done

  # [epagesj]

  write_epagesj_value epagesj server EPagesJServer
  write_epagesj_value epagesj host "${host[0]}"
  write_epagesj_value epagesj port "${port[0]}"
  write_epagesj_value epagesj masterUrl "http://${socket[1]}/solr/"
  write_epagesj_value epagesj slaveUrls "http://${socket[2]}/solr/"

  # [epages-solr]

  write_epagesj_value epages-solr server SolrServer
  write_epagesj_value epages-solr host "${host[1]}"
  write_epagesj_value epages-solr port "${port[1]}"

  # [epages-solr-slave]

  write_epagesj_value epages-solr-slave server SolrServer
  write_epagesj_value epages-solr-slave slave true
  write_epagesj_value epages-solr-slave host "${host[2]}"
  write_epagesj_value epages-solr-slave port "${port[2]}"
  write_epagesj_value epages-solr-slave masterUrl "http://${socket[1]}/solr/"
  write_epagesj_value epages-solr-slave solrDirPrefix "slave_${port[2]}"

  # [Mongo]

  write_epagesj_value Mongo hosts "${socket[3]}"

  EPAGES_INSTALL_MONGO_HOST=${host[3]} EPAGES_INSTALL_MONGO_PORT=${port[3]} update_mongod_conf
}

#..............................................................................
# findCommandLineArguments
#..............................................................................
findCommandLineArguments() {
  local USAGE='
syntax: findCommandLineArguments SECTION EPAGES_CONF
description:
  Find key commandLineArguments in SECTION of EPAGES_CONF
  Returns success (0) if found, else failure (1)
'
  [[ $# = 2 ]] || { echo "$USAGE" 1>&2; return 1; }

  local SECTION=$1
  local EPAGES_CONF=$2
  [[ -f "$EPAGES_CONF" ]] || return 1

  unset SECTION_FOUND KEY_FOUND
  while read line ; do
    if echo "$line" | grep -q '^\['$SECTION'\]' ; then
      SECTION_FOUND=1 ; continue
    fi
    # continue if other section than $SECTION
    [[ -z $SECTION_FOUND ]] && continue
    # continue if other section found
    if echo "$line" | grep -q '^\[' ; then
      unset SECTION_FOUND ; continue
    fi
    # break if key found
    if echo "$line" | grep -q commandLineArguments ; then
      KEY_FOUND=1 ; break
    fi
  done < "$EPAGES_CONF"

  [[ -n $KEY_FOUND ]] && return 0 || return 1
}

#..............................................................................
# update_epagesj_conf
#..............................................................................
update_epagesj_conf() {
  local USAGE='
syntax: update_epagesj_conf
description:
  Transfer customized value in old epagesj.conf to new epagesj.conf.
  If old epagesj.conf does not exist do not change anything.
  If old epagesj.sample.conf does not exist transfer all values.
  New files are stored in j/conf, old files in j/rpmsave/conf/.
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  EPAGESJ_CONFIG="${EPAGESJ_CONFIG:-/srv/epages/eproot/j/conf}"
  local CONF="$EPAGESJ_CONFIG"/epagesj.conf
  local OLD_CONF="${EPAGESJ:-/srv/epages/eproot/j}"/rpmsave/conf/epagesj.conf
  local OLD_SAMPLE=${OLD_CONF%.conf}.sample.conf

  [[ -f "$CONF" ]]     || return  # need file to write new values
  [[ -f "$OLD_CONF" ]] || return  # need file to get old values

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  # copy old values to new file
  for section in $($INI_CONFIG -f "$OLD_CONF" -G) ; do
    # for each old section get old keys/values
    $INI_CONFIG -f "$OLD_CONF" -s $section -G | sed "s,=, ,;s,^,$section ,"
  done | while read newsection key value ; do
    [[ -z $value ]] && continue
    unset sample_value
    if [[ -f "$OLD_SAMPLE" ]] ; then
      sample_value=$($INI_CONFIG -f "$OLD_SAMPLE" -s $newsection -k $key -G)
      if [[ -z $sample_value && $key = commandLineArguments ]] ; then
        # special handling of commandLineArguments:
        # if commandLineArguments does not appear in $newsection
        # in epages.sample.conf then use new value, not old value
        findCommandLineArguments $newsection "$OLD_SAMPLE" || continue
      fi
    fi

    # if sample value differs from old value
    # then transfer customized value to epagesj.conf
    [[ $sample_value = $value ]] && continue
    [[ "$(eval echo $value)" = "$(eval echo $sample_value)" ]] && continue
    $INI_CONFIG -f "$CONF" -s $newsection -k $key -v "$value" -i
  done
}

#..............................................................................
# update_epagesj_conf_xml
#..............................................................................
update_epagesj_conf_xml() {
  local USAGE='
syntax: update_epagesj_conf_xml
description:
  Copy old XML configuration files from j/rpmsave/* to j/*
  if old XML contains string "CUSTOMIZED CONFIGURATION FILE".
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  pushd "${EPAGESJ:-/srv/epages/eproot/j}"/rpmsave >/dev/null 2>&1 || return

  find * -type f -name '*.xml' | while read xml ; do
    grep -q 'CUSTOMIZED CONFIGURATION FILE' "$xml" || continue
    mkdir -m 755 -p ../$(dirname "$xml")
    cp -p $xml ../$(dirname "$xml")
  done

  popd >/dev/null
}

#..............................................................................
# epagesj_drop_cores
#..............................................................................
epagesj_drop_cores() {
  local USAGE='
syntax: epagesj_drop_cores [--base] [-b TARGET_VERSION]
description:
  checks whether or not to delete j/cores depending on EPAGESJ_DROP_CORES
  and EPAGESJ_DROP_CORES_FOR_VERSION from /var/epages/functions/config.sh
  and EPAGESJ_KEEP_CORES from environment.
  if EPAGESJ_KEEP_CORES is set to 0 -> j/cores shall be dropped
  if EPAGESJ_KEEP_CORES is set otherwise -> j/cores shall be kept
  if --base is used do not check EPAGESJ_DROP_CORES_FOR_VERSION
returns: success (0) - j/cores shall be dropped
         failure (1) - j/cores shall be kept
'
  local TARGET_VERSION BASE
  [[ $1 = --base ]] && { BASE=1 ; shift; }
  [[ $1 = -b ]] && { TARGET_VERSION=$2 ; shift 2; }
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  # if EPAGESJ_KEEP_CORES is set to 0 -> drop j/cores/
  [[ $EPAGESJ_KEEP_CORES = 0 ]] && return 0
  # if EPAGESJ_KEEP_CORES is set -> keep j/cores/
  [[ -n $EPAGESJ_KEEP_CORES ]] && return 1
  # if EPAGESJ_DROP_CORES is unset -> keep j/cores/
  [[ -z "$EPAGESJ_DROP_CORES$EPAGESJ_DROP_CORES_FOR_VERSION" ]] && return 1

  local EPAGESJ_VERSION=$(pkg_installed -V j | tail -1)
  # if epages-j is not found (initial installation) -> drop j/cores/
  [[ -z $EPAGESJ_VERSION ]] && return 0

  # cut version after minus: 1.2.3-55 -> 1.2.3
  EPAGESJ_VERSION=${EPAGESJ_VERSION%%-*}

  if [[ -z $BASE ]] ; then
    for DROP_VERSION in $EPAGESJ_DROP_CORES_FOR_VERSION ; do
      DROP_VERSION=${DROP_VERSION// /}
      # go on if $DROP_VERSION is not a version string
      [[ -n ${DROP_VERSION//[0-9.-]/} ]] && continue
      # if installed version == $DROP_VERSION -> drop j/cores/
      [[ $EPAGESJ_VERSION = $DROP_VERSION ]] && return 0
    done
  fi

  for DROP_VERSION in $EPAGESJ_DROP_CORES ; do
    DROP_VERSION=${DROP_VERSION// /}
    # go on if $DROP_VERSION is not a version string
    [[ -n ${DROP_VERSION//[0-9.-]/} ]] && continue
    # go on if $DROP_VERSION > $TARGET_VERSION
    test_version $DROP_VERSION -gt ${TARGET_VERSION:-0} && continue
    # if installed version < $DROP_VERSION -> drop j/cores/
    test_version $EPAGESJ_VERSION -lt $DROP_VERSION && return 0
  done
  # otherwise -> keep j/cores/
  return 1
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
