#!/bin/bash

#==============================================================================
# basic operating system functions
#==============================================================================

fun_linux_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export pathmunge is_positive repair_number repair_name
export set_stat backup_file initial_file chkconfig_do
export extract_host_from_url extract_port_from_url

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# chkconfig_do
#..............................................................................

chkconfig_do() {
  local USAGE='
syntax: chkconfig_do {--add|--del|--list} SERVICE(S)
description: runs either sysv-rc-conf (debian) or chkconfig (else)
'
  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }
  local DO=$1 ; shift
  case $EPAGES_KERNEL in
    *d) case $DO in
          --add)  for i in $@ ; do PERL5LIB= /usr/sbin/update-rc.d $i defaults ; done ;;
          --del)  for i in $@ ; do PERL5LIB= /usr/sbin/update-rc.d -f $i remove ; done ;;
          --list) for i in $@ ; do PERL5LIB= /usr/sbin/sysv-rc-conf $DO $i ; done ;;
          *) echo "$USAGE" 1>&2; return 1 ;;
        esac
        ;;
     *) for i in $@ ; do 
          local FPATH=$(systemctl --property=FragmentPath show $i 2>/dev/null)
          case $DO in
            --add)  if [[ -n ${FPATH#FragmentPath=} ]] ; then
                      systemctl enable ${FPATH##*/} 
                    else
                      PERL5LIB= /sbin/chkconfig $DO $i
                    fi ;;
            --del)  if [[ -n ${FPATH#FragmentPath=} ]] ; then
                      systemctl disable ${FPATH##*/} 
                    else
                      PERL5LIB= /sbin/chkconfig $DO $i
                    fi ;;
            --list) if [[ -n ${FPATH#FragmentPath=} ]] ; then
                      systemctl list-unit-files | grep ${FPATH##*/}
                    else
                      PERL5LIB= /sbin/chkconfig $DO $i
                    fi ;;
            *) echo "$USAGE" 1>&2; return 1 ;;
          esac
        done
        ;;
  esac
}

#..............................................................................
# pathmunge
#..............................................................................

pathmunge() {
  local USAGE='
syntax: pathmunge [SOURCE_PATH] TARGET_PATH
  TARGET_PATH: semicolon-separated path to change
  SOURCE_PATH: semicolon-separated path to prepend
examples:
  PATH=$(pathmunge /usr/bin:/bin:/usr/bin) -> /usr/bin:/bin
  PATH=$(pathmunge /usr/bin /bin:/usr/bin) -> /bin:/usr/bin
  PATH=$(pathmunge /usr/bin /bin)          -> /usr/bin:/bin
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# -eq 0 ]] && return

  local IN=$2
  local ALL=${IN:+:$IN:}
  local OUT
  local OIFS=$IFS
  IFS=:
  for i in $1 ; do
    [ -z "$i" ] && continue
    test -d "$i" || continue
    [ "${ALL%:$i:*}" = "${ALL#*:$i:}" ] || continue
    ALL="$ALL:$i"
    OUT="$OUT:$i"
  done
  IFS=$OIFS
  [[ -n $IN && -z $OUT ]] && ALL=$IN
  [[ -n $IN && -n $OUT ]] && ALL="${OUT#:}:$IN"
  echo "${ALL#:}"
}

#..............................................................................
# is_positive
#..............................................................................
is_positive() {
  local USAGE='
syntax: is_positive NUMBER
returns: 0 - if 1 or bigger
         1 - else
'
  [[ $1 != ${1#-} && $1 = ${1#-[0-9]} ]] && { echo "$USAGE" 1>&2; return 1; }

  # bash can't compute numbers >= 1024*1024*1024*1024*1024*1024*8
  [[ -z $1 || -n ${1//[0-9-]/} || ${#1} -gt 18 ]] && return 1
  return $(($1 < 1))
}

#..............................................................................
# repair_number
#..............................................................................
repair_number() {
  local USAGE='
syntax: repair_number STRING [RANGE] [DEFAULT]
example: repair_number ST4556RING 1..1000 500
example: repair_number ST4556RING 1000.. 5000
description: returns all positive numbers in STRING
  - if number is not in RANGE, returns DEFAULT (or nothing)
  - RANGE is MIN..MAX where MIN and MAX are optional, default 1..65535
  - all numbers must be within range 0..1024*1024*1024*1024*1024*1024*8
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local ID=${1//[^0-9-]/} ; shift
  local DEFAULT RANGE MIN MAX

  if [[ -n $1 && $1 = ${1#*..} ]] ; then
    DEFAULT=$1 ; RANGE=$2
  elif [[ -n $1 ]] ; then
    DEFAULT=$2 ; RANGE=$1
  fi
  if [[ -n $RANGE ]] ; then
    [[ $RANGE != ${RANGE%%..*} ]] && MIN=${RANGE%%..*}
    [[ $RANGE != ${RANGE##*..} ]] && MAX=${RANGE##*..}
  fi
  MIN=${MIN:-1}
  MAX=${MAX:-65535}

  # bash can't compute numbers >= 1024*1024*1024*1024*1024*1024*8
  [[ ${#DEFAULT} -gt 18 ]] && { echo -e "ERROR: DEFAULT too big$USAGE" 1>&2; return 1; }
  [[ ${#MIN} -gt 18 ]] && { echo -e "ERROR: MIN too big$USAGE" 1>&2; return 1; }
  [[ ${#MAX} -gt 18 ]] && { echo -e "ERROR: MAX too big$USAGE" 1>&2; return 1; }
  [[ $MIN -gt $MAX ]] && { echo -e "ERROR: MIN greater than MAX$USAGE" 1>&2; return 1; }

  [[ -z $ID || ${#ID} -gt 18 ]] && { [[ -n $DEFAULT ]] && echo $DEFAULT; return; }
  [[ $ID -lt $MIN || $ID -gt $MAX ]] && { [[ -n $DEFAULT ]] && echo $DEFAULT; return; }
  [[ -n $ID ]] && echo $ID
}

#..............................................................................
# repair_name
#..............................................................................
repair_name() {
  local USAGE='
syntax: repair_name STRING
description: returns "repaired" STRING:
  - remove all characters not in [A-Za-z0-9_.-]
  - remove leading [.-]
  - trim STRING up to 32 characters
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # remove all characters not in [A-Za-z0-9_.-]
  local ID=$(echo "$1" | tr -d -c 'A-Za-z0-9_.-')
  # remove leading [.-]
  while [[ $ID != ${ID/#[.-]/} ]] ; do ID=${ID/#[.-]/} ; done
  # name name must not have more than 32 characters
  [[ -n $ID ]] && echo ${ID:0:32}
}

#..............................................................................
# set_stat
#..............................................................................
set_stat () {
  local USAGE='
syntax: set_stat -r REFERENCE_FILE FILE
description: sets user/group/mode of FILE as in REFERENCE_FILE
'
  [[ $# -ne 3 || $1 != -r ]] && { echo "$USAGE" 1>&2; return 1; }

  local REFERENCE_FILE=$2
  local EMPTY_FILE=$3

  ls $REFERENCE_FILE >/dev/null || { echo "$USAGE" 1>&2; return 1; }
  touch "$EMPTY_FILE" || return 1

  local MODE=$(stat -c '%a' $REFERENCE_FILE)
  [[ -d $REFERENCE_FILE ]] && MODE=$(echo "ibase=10; obase=8; $((0$MODE & 0666))" | bc)

  chmod $MODE "$EMPTY_FILE"
  chown $(stat -c '%u:%g' $REFERENCE_FILE) "$EMPTY_FILE"
  return 0
}

#..............................................................................
# backup_file
#..............................................................................
backup_file () {
  local USAGE='
syntax: backup_file FILE
description:
  If file ${FILE%.ep[tx]*} exists, copy ${FILE%.ep[tx]*} to ${FILE%.ep[tx]*}.~$EPOCH~
  Else if FILE exists and has suffix .ep[tx], copy FILE to ${FILE%.ep[tx]*}
'
  [[ $# -ne 1 ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILE=${1%.ep[tx]*}

  if [[ -f $FILE ]] ; then
    # backup file
    cp -p "$FILE" "$FILE".~$(date "+%s")~ || return 1
  elif [[ -f $1 && $1 != $FILE ]] ; then
    # copy FILE.ept to FILE
    cp -p "$1" "$FILE" || return 1
  else
    return 1
  fi
  return 0
}

#..............................................................................
# initial_file
#..............................................................................
initial_file () {
  local USAGE='
syntax: initial_file DIRECTORY
description: for all FILE*.initial in DIRECTORY, copy it to ${FILE%.initial}
  (unless exists)
'
  [[ $# -ne 1 ]] && { echo "$USAGE" 1>&2; return 1; }
  pushd "$1" >/dev/null || { echo "$USAGE" 1>&2; return 1; }

  for i in $(find . -type f -name '*.initial') ; do
    [ ! -f "${i%.initial}" ] && cp -p "$i" "${i%.initial}" || {
      [ "${i##*/}" = version.txt.initial ] && cat "$i" >> version.txt
    }
  done
  popd >/dev/null
}

#..............................................................................
# extract_host_from_url
#..............................................................................
extract_host_from_url () {
  local USAGE='
syntax: extract_host_from_url URL
description: extracts the host part of the URL
'
  [[ $# -ne 1 ]] && { echo "$USAGE" 1>&2; return 1; }
  local SOCKET=$(echo $1 | sed -r 's,^\w+://\[?([^]/]+).*,\1,')
  echo ${SOCKET%%:*}
}

#..............................................................................
# extract_port_from_url
#..............................................................................
extract_port_from_url () {
  local USAGE='
syntax: extract_port_from_url URL
description: extracts the port part of the URL
'
  [[ $# -ne 1 ]] && { echo "$USAGE" 1>&2; return 1; }
  local SOCKET=$(echo $1 | sed -r 's,^\w+://\[?([^]/]+).*,\1,')
  [[  $SOCKET = ${SOCKET##*:} ]] || echo ${SOCKET##*:}
}


#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
