#!/bin/bash

#==============================================================================
# MySQL
#==============================================================================

fun_mysql_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export my_host

[[ -n $fun_account_sourced ]] || . $(dirname $BASH_SOURCE)/fun-account.sh
export add_user get_user_info get_group_info

[[ -n $fun_pkg_sourced ]] || . $(dirname $BASH_SOURCE)/fun-pkg.sh
export show_rpm

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export mysqld_name my_cnf mysql_installed mysqld_installed check_mysqld_rpm
export get_mysql_option get_mysql_connection set_mysql_connection
export create_mysql_directories add_innodb_data_file_path has_mariadb_deb
export check_mysql_libs_rpm

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# mysqld_name
#..............................................................................
mysqld_name() {
  local USAGE='
syntax: mysqld_name
returns: name of MySQL daemon
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local FPATH=$(systemctl --property=FragmentPath show mariadb 2>/dev/null)
  [[ -n ${FPATH#FragmentPath=} ]] && { echo mariadb; return; }

  [[ -f /etc/init.d/mysql ]] && { echo mysql; return; }
  [[ -f /etc/init.d/mysqld ]] && { echo mysqld; return; }
}

#..............................................................................
# my_cnf
#..............................................................................
my_cnf() {
  local USAGE='
syntax: my_cnf [$EPAGES_CONFIG/Database.d/SOME.conf]
returns: name of MySQL configuration file (from MySQLConfigFile=...) (if exist)
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # mariadb has server config in /etc/my.cnf.d/server.cnf
  local MY_CNF=/etc/my.cnf.d/server.cnf
  # other mysqld have server config in /etc/(mysql/)?my.cnf
  [[ ! -f $MY_CNF && -f /etc/my.cnf ]] && MY_CNF=/etc/my.cnf
  [[ ! -f $MY_CNF && -f /etc/mysql/my.cnf ]] && MY_CNF=/etc/mysql/my.cnf
  
  [[ $# = 0 ]] && { [[ -f $MY_CNF ]] && echo $MY_CNF; return; }
  ls -d $1 >/dev/null || { echo "$USAGE" 1>&2; return 1; }

  local BU_MY_CNF=$(grep '^\s*MySQLConfigFile\s*=' $1 \
    | sed 's/.*MySQLConfigFile[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
  [[ -f ${BU_MY_CNF:-$MY_CNF} ]] && echo ${BU_MY_CNF:-$MY_CNF}
}

#..............................................................................
# mysql_installed
#..............................................................................
mysql_installed() {
  local USAGE='
syntax: mysql_installed
returns: 0 - MySQL client is installed ; 1 - not
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  cat $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null \
  | grep -Pq '^\s*datasource\s*=\s*dbi:mysql' \
  && [[ -x ${EPSERVICE:-/srv/epages/eproot/Service/epagesService} ]]
  return $?
}

#..............................................................................
# mysqld_installed
#..............................................................................
mysqld_installed() {
  local USAGE='
syntax: mysqld_installed
returns: 0 - MySQL server is installed ; 1 - not
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local file
  for file in $(ls $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null) ; do
    datasource=$(grep -P '^\s*datasource\s*=\s*dbi:mysql' $file \
      | sed 's/.*datasource[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
    [[ -z $datasource ]] && continue
    [[ $datasource = ${datasource#*:*:*} ]] && continue
    host=${datasource##*hostname=}
    host=${host##*host=}
    if [[ $host != $datasource ]] ; then
      # is internet socket
      host=${host%%;*}
      my_host $host && return 0
    elif [[ $datasource != ${datasource#dbi:mysql:mysql_socket} ]] ; then
      # is file socket
      [[ -n $(my_cnf $file) ]] && return 0
    fi
  done
  return 1
}

. /etc/sysconfig/epages6

#..............................................................................
# get_mysql_option
#..............................................................................
get_mysql_option() {
  local USAGE='
syntax: [EPAGES_INSTALL_MY_CNF=...] get_mysql_option ["SECTIONS" [KEY DEFAULT]]
description: echoes value of first occurence of KEY in SECTIONS from file my.cnf
  (or EPAGES_INSTALL_MY_CNF if defined), else DEFAULT
  - if KEY is missing shows all key=value pairs of "SECTIONS"
  - if "SECTIONS" is missing shows all key=value pairs of all section
examples:
  get_mysql_option
  get_mysql_option "server mysqld"
  get_mysql_option "server mysqld" datadir /var/lib/mysql
  get_mysql_option "client mysql server mysqld" port 3306
'
  [[ $# -gt 3 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local MY_CNF_ARG result
  local SECTIONS=${1:-"client mysql mysqld_safe mysqld server mysql_server mysql.server"}
  local KEY=$2
  local DEFAULT=$3

  [[ -n $EPAGES_INSTALL_MY_CNF ]] && MY_CNF_ARG="-c $EPAGES_INSTALL_MY_CNF"

  if [[ -z $KEY ]] ; then
    if type -p my_print_defaults >/dev/null ; then
      for section in $SECTIONS ; do
        my_print_defaults $MY_CNF_ARG $section
      done
    else
      local INI_CONFIG=/var/epages/ini-config.sh
      local MY_CNF=${EPAGES_INSTALL_MY_CNF:-$(my_cnf)}
      [[ -f $MY_CNF && -x $INI_CONFIG ]] || return
      for section in $SECTIONS ; do
        $INI_CONFIG -f $MY_CNF -s $section -G | grep -P '^\s*\w.*=.*/' | sed 's,^,--,'
      done
    fi
    return
  fi

  type -p my_print_defaults >/dev/null || { echo -n $DEFAULT; return; }

  for section in $SECTIONS ; do
    result=$(my_print_defaults $MY_CNF_ARG $section | sed -n "s/^--$KEY=//p" | tail -n 1)
    [[ -n $result ]] && { echo -n $result; return; }
  done
  echo -n $DEFAULT
}

#..............................................................................
# create_mysql_directories
#..............................................................................
create_mysql_directories() {
  local USAGE='
syntax: create_mysql_directories
description: creates all directories defined in MySQL options
  (does nothing if MySQL client is not installed)
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  # do nothing if mysql client is (apparently) not installed
  type -p my_print_defaults >/dev/null || [[ -n $(my_cnf) ]] || return

  add_user -u MYSQL -g MYSQL
  local MYSQL_USER=$(get_user_info MYSQL) ; MYSQL_USER=${MYSQL_USER%%:*}
  local MYSQL_GROUP=$(get_group_info MYSQL) ; MYSQL_GROUP=${MYSQL_GROUP%%:*}

  local SELINUX_ENFORCING=1
  if ! chcon --version >/dev/null 2>&1 ; then
    SELINUX_ENFORCING=
  elif ! type -p getenforce >/dev/null ; then
    SELINUX_ENFORCING=
  elif [[ "$(getenforce | tr A-Z a-z)" != enforcing ]] ; then
    SELINUX_ENFORCING=
  fi

  get_mysql_option | while IFS== read KEY DIRECTORY ; do
    [[ $DIRECTORY =~ / ]] || continue
    if [[ -n $SELINUX_ENFORCING && -e $DIRECTORY ]] ; then
      # fix selinux context for existing files (from other mysql than 5.6)
      chcon -h unconfined_u:object_r:mysqld_db_t:s0 $DIRECTORY 2>/dev/null
      chcon -L unconfined_u:object_r:mysqld_db_t:s0 $DIRECTORY 2>/dev/null
    fi
    [[ $KEY =~ dir$ ]] || DIRECTORY=$(dirname "$DIRECTORY")
    [[ -d $DIRECTORY ]] && continue
    install -g $MYSQL_GROUP -o $MYSQL_USER -m 755 -d $DIRECTORY
  done
}

#..............................................................................
# get_mysql_connection
#..............................................................................
get_mysql_connection() {
  local USAGE='
syntax: MYSQL_CONN=($(get_mysql_connection))
description: echoes MySQL host and port derived from /etc/sysconfig/epages6 and my.cnf
example:
  MYSQL_CONN=($(get_mysql_connection)) ; HOST=${MYSQL_CONN[0]} ; PORT=${MYSQL_CONN[1]}
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local MYSQL_CONN=${EPAGES_MYSQL_CONNECTION:-$EPAGES_INSTALL_MYSQL_CONNECTION}
  # MYSQL_CONN can be either 'host;port' or 'host=host;port=port'
  MYSQL_CONN=$(echo $MYSQL_CONN | sed -r 's,[a-z]+=,,g')
  MYSQL_CONN=${MYSQL_CONN:-$EPAGES_INSTALL_MYSQL_HOST}

  local MYSQL_IP=$(my_ips | grep '\.' | head -1)
  local MYSQL_HOST=${EPAGES_INSTALL_MYSQL_HOST:-${MYSQL_CONN%;*}}
  MYSQL_HOST=$(get_mysql_option "client mysql" host ${MYSQL_HOST:-$MYSQL_IP})

  local MYSQL_PORT=${MYSQL_CONN#*;}
  [[ $MYSQL_CONN = $MYSQL_PORT ]] && MYSQL_PORT=
  MYSQL_PORT=$(get_mysql_option "client mysql server mysqld" port ${MYSQL_PORT:-3306})

  echo $MYSQL_HOST $MYSQL_PORT
}

#..............................................................................
# set_mysql_connection
#..............................................................................
set_mysql_connection() {
  local USAGE='
syntax: set_mysql_connection BU HOST PORT
description: sets dbi:mysql:host=HOST[;port=PORT] in \$EPAGES_CONFIG/Database.d/BU.conf
example:
  set_mysql_connection ASPoolDB $(get_mysql_connection)
'
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local DB_CONF="$EPAGES_CONFIG/Database.d/$1.conf"
  ls $DB_CONF >/dev/null || { echo "ERROR in: set_mysql_connection $@"; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local MYSQL_CONN="host=$2"
  [[ ${3:-3306} = 3306 ]] || MYSQL_CONN="$MYSQL_CONN;port=$3"

  $INI_CONFIG -f $DB_CONF -s Connection -k datasource -v "dbi:mysql:$MYSQL_CONN" -i
  chown ep_appl:epages $DB_CONF
}

#..............................................................................
# check_mysqld_rpm
#..............................................................................
check_mysqld_rpm() {
  local USAGE='
syntax: check_mysqld_rpm REQUIRED_VERSION_FAMILY [EPAGES_RPM]
returns: success (if up-to-date or not installed at all) or failure (else)
example: check_mysqld_rpm 5.6
example: check_mysqld_rpm 5.6 epages-mysqld
'
  [[ $# -gt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local myfamily=$1
  local EPAGES_RPM=$2
  local myvendor='Oracle and/or its affiliates'
  local myvendor_2='Sun Microsystems, Inc.'
  local myvendor_3='MySQL AB'

  [[ $myfamily =~ ^[0-9]*.[0-9]*$ ]] || { echo "$USAGE" 1>&2; return 1; }

  # get installed MySQL server RPM
  local installed=
  for i in mysql MySQL mariadb ; do
    installed=$(rpm -q --whatprovides $i-server 2>/dev/null)
    [[ $? -eq 0 && -n $installed ]] && break
    installed=
  done

  # return success if not installed
  [[ -z $installed ]] && return 0

  # get information from installed MySQL server RPM
  installed=$(echo $installed | sed -r 's/([^ ]*) .*/\1/')
  local vendor=$(rpm -q --qf '[%{VENDOR}]' "$installed" 2>/dev/null)
  vendor=${vendor:-'<unknown>'}
  local version=$(rpm -q --qf '[%{VERSION}]' "$installed" 2>/dev/null)
  local family=$(echo $version | sed -r 's,^([0-9]+\.[0-9]+)\..*,\1,')
  [[ $family =~ ^[0-9]*.[0-9]*$ ]] || family="<unrecognized version $version>"
  local PACKAGES_TO_REMOVE=$(rpm -q --qf '[%{NAME}]' "$installed" 2>/dev/null)
  echo $version

  # check if up-to-date
  local error_text=
  if [[ $vendor != $myvendor_3 && $vendor != $myvendor_2 && $vendor != $myvendor ]] ; then
    error_text="$error_text
The current MySQL server package is provided by a different
vendor ($vendor) than $myvendor_3, $myvendor_2, or $myvendor.
Some files may be installed to different locations, including log
files and the service startup script in /etc/init.d/.
"
  fi
  if [ "$family" != "$myfamily" ]; then
    error_text="$error_text
Upgrading directly from MySQL $family to MySQL $myfamily may not
be safe in all cases.  A manual dump and restore using mysqldump is
recommended.  It is important to review the MySQL manual's Upgrading
section for version-specific incompatibilities.
"
  fi

  # return success if up-to-date
  [[ -z $error_text ]] && return 0

  # get error message content

  if [[ -z $PACKAGES_TO_REMOVE ]] ; then
    PACKAGES_TO_REMOVE='<mysql packages>'
  elif [[ $PACKAGES_TO_REMOVE = mariadb-server ]] ; then
    rpm -q --quiet mariadb && PACKAGES_TO_REMOVE='mariadb mariadb-server'
  fi
  for rpm in $(show_rpm mysql mysql-libs) ; do
    PACKAGES_TO_REMOVE="$PACKAGES_TO_REMOVE
   rpm -e --nodeps $rpm"
  done

  local SHUTDOWN_MYSQL
  if [[ -n $(mysqld_name) ]] ; then
    SHUTDOWN_MYSQL=":

   service $(mysqld_name) stop"
  fi

  local UPDATE_EPAGES
  if [[ $EPAGES_RPM = epages-setup ]] ; then
    UPDATE_EPAGES="
7. Update the ePages package:

   yum -y install epages --enablerepo epages-setup
"
  elif rpm -q --quiet epages-mysqld || [[ $EPAGES_RPM = epages-mysqld ]] ; then
    UPDATE_EPAGES="
7. Update the ePages package epages-mysqld:

   yum -y install epages-mysqld epages-mysql --enablerepo epages-setup
"
  elif rpm -q --quiet epages-mysql || [[ $EPAGES_RPM = epages-mysql ]] ; then
    UPDATE_EPAGES="
7. Update the ePages package epages-mysql:

   yum -y install epages-mysql --enablerepo epages-setup
"
  fi

  local START_EPAGES
  if [[ -n $UPDATE_EPAGES ]] ; then
    START_EPAGES="
0. Stop ePages services, run on any ePages host:

   service epages6 stop
"
    UPDATE_EPAGES="$UPDATE_EPAGES
8. Start ePages services, run on any ePages host:

   service epages6 start
"
  fi

  local MYSQL_SECTIONS='mysqld server mysql_server mysql.server mysqld_safe client mysql'
  local MYSQL_DATADIR=$(get_mysql_option "$MYSQL_SECTIONS" datadir /var/lib/mysql)

  add_user -u MYSQL -g MYSQL
  local MYSQL_USER=$(get_user_info MYSQL) ; MYSQL_USER=${MYSQL_USER%%:*}
  local MYSQL_GROUP=$(get_group_info MYSQL) ; MYSQL_GROUP=${MYSQL_GROUP%%:*}
  install -g $MYSQL_GROUP -o $MYSQL_USER -m 755 -d $MYSQL_DATADIR

  # write error message

  local ERROR_MESSAGE="

##############################################################################
# NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE #

A MySQL server package ($installed) is installed.
$error_text
A manual upgrade is required.
$STOP_EPAGES
1. Create a complete, working backup of your data and my.cnf files.

2. Shut down the MySQL server cleanly$SHUTDOWN_MYSQL

3. Remove the existing MySQL packages:

   yum -y remove $PACKAGES_TO_REMOVE

4. Install the new MySQL server supplied by $myvendor:

   yum -y install MySQL-server

5. Start the new MySQL server:

   /etc/init.d/mysql start

6. Run the 'mysql_upgrade' program:

   /usr/bin/mysql_upgrade -u root [-p PASSWORD]
$UPDATE_EPAGES
Read this brief description of the upgrade process in file
$MYSQL_DATADIR/MYSQL-UPGRADE-$myfamily.README

Further reading can be found in the MySQL manual, in the Upgrading section.

# NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE #
##############################################################################

"

  local SHORT_ERROR_MESSAGE="

##############################################################################
#  ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR   #

EPAGES CAN'T BE INSTALLED BECAUSE OUTDATED MYSQL SERVER PACKAGE FOUND:

$installed

A MANUAL UPGRADE IS REQUIRED.

READ THE DESCRIPTION OF THE UPGRADE PROCESS IN FILE:

$MYSQL_DATADIR/MYSQL-UPGRADE-$myfamily.README

#  ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR   #
##############################################################################

"

  local README=$MYSQL_DATADIR/MYSQL-UPGRADE-$myfamily.README
  local UPDATE_README
  case $EPAGES_RPM in
    epages-setup) UPDATE_README=1 ;;
    epages-mysqld) [[ -f $README-part-epages-setup ]] || UPDATE_README=1 ;;
    epages-mysql) [[ -f $README-part-epages-setup || -f $README-part-epages-mysqld ]] || UPDATE_README=1 ;;
  esac
  EPAGES_RPM=${EPAGES_RPM:-other}

  echo "$ERROR_MESSAGE" > $README-part-$EPAGES_RPM
  [[ -n $UPDATE_README ]] && /bin/cp -pf $README-part-$EPAGES_RPM $README
  echo "$SHORT_ERROR_MESSAGE" 1>&2

  # return failure because outdated
  return 1
}

#..............................................................................
# has_mariadb_deb
#..............................................................................
has_mariadb_deb() {
  local USAGE='
syntax: has_mariadb_deb
returns: checks if MariaDB server or client is installed (success) or not (failure)
example: has_mariadb_deb
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  if dpkg-query -l 'mariadb-server*' >/dev/null 2>&1 ; then
cat <<HERE
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MariaDB server found but ePages requires Oracle MySQL 5.6.
Before continuing with ePages installation you must upgrade
to Oracle MySQL 5.6 by executing:

  apt-get -y update
  apt-get -y install mysql-community-server mysql-community-client
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
HERE
  elif dpkg-query -l 'mariadb-client*' >/dev/null 2>&1 ; then
cat <<HERE
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
MariaDB client found but ePages requires Oracle MySQL 5.6.
Before continuing with ePages installation you must upgrade
to Oracle MySQL 5.6 by executing:

  apt-get -y update
  apt-get -y install mysql-community-client
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
HERE
  else
    # return failure unless MariaDB server/client found
    return 1
  fi

  # return success if MariaDB server/client found
  return 0
}

#..............................................................................
# check_mysql_libs_rpm
#..............................................................................
check_mysql_libs_rpm() {
  local USAGE='
syntax: MYSQL_LIBS=$(check_mysql_libs_rpm)
returns: name of mysql-libs RPMs
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  rpm -q --quiet mysql-libs || return 

  for mysql_libs in $(rpm -q mysql-libs) ; do
     echo $mysql_libs
  done
}

#..............................................................................
# add_innodb_data_file_path
#..............................................................................

add_innodb_data_file_path() {
  local USAGE='
syntax: add_innodb_data_file_path
description: adds innodb_data_file_path=ibdata1:10M:autoextend unless key exists
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  add_user -u MYSQL -g MYSQL
  local INI_CONGIG=/var/epages/ini-config.sh
  [[ -x $INI_CONGIG ]] || return 1

  local MYSQL_CNF=${EPAGES_INSTALL_MY_CNF:-$(my_cnf)}
  MYSQL_CNF=${MYSQL_CNF:-/etc/my.cnf}
  [[ -f $MYSQL_CNF ]] && /bin/cp -p $MYSQL_CNF $MYSQL_CNF.epages-backup

  local MYSQL_USER=$(get_user_info MYSQL) ; MYSQL_USER=${MYSQL_USER%%:*}
  local MYSQL_GROUP=$(get_group_info MYSQL) ; MYSQL_GROUP=${MYSQL_GROUP%%:*}

  if [[ -f /etc/my.cnf ]] ; then
    # if /etc/my.cnf exists
    for mysql_libs in $(check_mysql_libs_rpm) ; do
      # and is provided by mysql-libs
      rpm -q $mysql_libs -l
    done | grep -q '^/etc/my.cnf$' && {
      # move /etc/my.cnf to /etc/mysql/my.cnf
      # because rpm -e --nodeps mysql-libs deletes /etc/my.cnf
      install -g $MYSQL_GROUP -o $MYSQL_USER -m 755 -d /etc/mysql
      mv /etc/my.cnf /etc/mysql
      MYSQL_CNF=/etc/mysql/my.cnf
    }
  fi

  touch $MYSQL_CNF || return 1
  local ENGINE=$($INI_CONGIG -f $MYSQL_CNF -s mysqld -k default-storage-engine -G)
  [[ $ENGINE = InnoDB ]] || {
    $INI_CONGIG -f $MYSQL_CNF -s mysqld -k default-storage-engine -v InnoDB -i
  }
  $INI_CONGIG -f $MYSQL_CNF -s mysqld -k innodb_data_file_path -G | grep -q . || {
    $INI_CONGIG -f $MYSQL_CNF -s mysqld -k innodb_data_file_path -v ibdata1:10M:autoextend -i
  }
  chown $MYSQL_USER:$MYSQL_GROUP $MYSQL_CNF
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
