#!/bin/bash

#==============================================================================
# edit repositories
#==============================================================================

fun_repoedit_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export get_cartridge_fullversion

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL EPAGES_ARCH
export INI_CONFIG=/var/epages/ini-config.sh

[[ -n $fun_java_sourced ]] || . $(dirname $BASH_SOURCE)/fun-java.sh
export test_version

[[ -n $fun_pkg_sourced ]] || . $(dirname $BASH_SOURCE)/fun-pkg.sh
export pkg_installed

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export get_arch_repo update_repo_arch get_update_repo
export update_repo_version update_repo_url
export update_yum_repo use_new_repos
export delete_obsolete_repos enable_repo
export link_yum2zypper unlink_yum2zypper
export refresh_repo_by_upgrade_path

#----------------------f--------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# get_arch_repo
#..............................................................................
get_arch_repo() {
  local USAGE='
syntax: get_arch_repo
returns: epages repo url which can be used as input for update_repo_url
operating system: does not work on debian/ubuntu
example: update_repo_url $(get_arch_repo) setup
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local repo
  local repofile=/etc/yum.repos.d/epages-arch.repo
  [[ ! -f $repofile && -f $repofile.rpmsave ]] && repofile=$repofile.rpmsave
  if [[ -f $repofile ]] ; then
    repo=$(grep -P '^\s*baseurl' $repofile | head -1 |
           tr -d '[\042\047[:space:]]' | sed 's,[^=]*=,,;s,/kernel/.*,,')
  fi

  if [[ -z $repo ]] ; then
    echo "baseurl not found in: $repofile" 1>&2
    return 1
  else
    echo $repo
    return 0
  fi
}

#..............................................................................
# update_repo_arch
#..............................................................................
update_repo_arch() {
  local USAGE='
syntax: update_repo_arch REPO1 REPO2...
description: sets arch (x86_64/i386) in REPO1 REPO2...
operating system: does not work on debian/ubuntu
example: update_repo_arch base arch setup
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local repo subst

  for repo in $@ ; do
    [[ -f /etc/yum.repos.d/epages-$repo.repo ]] || continue
    case $repo in
      base) subst="s,KERNEL,$EPAGES_KERNEL," ;;
      arch|setup) subst="s,ARCH,$EPAGES_ARCH,;s,KERNEL,$EPAGES_KERNEL," ;;
      *) continue ;; # other repos do not contain epages versions
    esac
    sed -ri "$subst" /etc/yum.repos.d/epages-$repo.repo
  done
}

#..............................................................................
# get_update_repo
#..............................................................................
get_update_repo() {
  local USAGE='
syntax: get_update_repo VERSION
description: gets repository from /etc/apt/sources.list.d/epages.list
operating system: works only on debian/ubuntu
example: get_update_repo
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] && return
  local EPAGES_LIST=/etc/apt/sources.list.d/epages.list
  [[ -f $EPAGES_LIST ]] || return

  egrep '/apt/([6-9](\.[0-9]+)+-)?updates' $EPAGES_LIST | head -1 \
  | sed -r 's,.*deb ,,;s,(.+)/apt/([6-9](\.[0-9]+)+-)?updates.*,\1,'
}

#..............................................................................
# update_repo_version
#..............................................................................
update_repo_version() {
  local USAGE='
syntax: update_repo_version VERSION
description: updates epages apt repo to VERSION
operating system: works only on debian/ubuntu
example: update_repo_version 6.16.4
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] && return

  # This regex updates the version in the repo path.
  # The complete line looks like this:
  # deb ftp://ftp.epages.com/pub/epages/build/repo/6.17.15/latest/apt/6.17.15-updates stable non-free
  local subst='s,/apt/([0-9.]+-)?((updates|hotfix)[[:space:]]),/apt/'$1'-\2,;s,/apt/[0-9.]+([[:space:]]),/apt/'$1'\1,'
  sed -ri "$subst" /etc/apt/sources.list.d/epages.list
}

#..............................................................................
# update_repo_url
#..............................................................................

update_repo_url() {
  local USAGE='
syntax: update_repo_url URL...
description: updates epages apt repo to URL
operating system: works only on debian/ubuntu
example: update_repo_url http://ep-server/yum/qa
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] && return

  local REPOURL=$1

  local subst='s, [^ ]+/apt/, '$REPOURL'/apt/,'
  sed -ri "$subst" /etc/apt/sources.list.d/epages.list

  # REPOHOST in http://REPOHOST/foobar
  local REPOHOST=${REPOURL#*://}
  # return if REPOURL has no ://
  [[ $REPOHOST = $REPOURL ]] && return

  local EPAGES_PREF=/etc/apt/preferences.d/epages-repo.pref
  mkdir -m 755 -p ${EPAGES_PREF%/*}
  /bin/echo -e "\nPackage: *\nPin: origin ${REPOHOST%%/*}\nPin-Priority: 703" > $EPAGES_PREF
  chmod 644 $EPAGES_PREF
}

#..............................................................................
# use_new_repos
#..............................................................................

use_new_repos() {
  local USAGE='
syntax: use_new_repos
description: moves .repo.rpmnew to .repo
operating system: does not work on debian/ubuntu
example: use_new_repos
'
  [[ $# -gt 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  pushd /etc/yum.repos.d >/dev/null 2>&1 || return

  for i in $(ls -d *.repo 2>/dev/null) ; do
    grep -q templateurl $i && continue
    grep -q templateurl $i.rpmnew 2>/dev/null || continue
    /bin/mv $i.rpmnew $i
  done
  for i in $(ls -d mirrors-epages-*.rpmnew 2>/dev/null) ; do
    /bin/mv $i ${i%.rpmnew}
  done

  popd >/dev/null
}

#..............................................................................
# update_yum_repo
#..............................................................................

update_yum_repo() {
  local USAGE='
syntax: update_yum_repo REPO1 REPO2...
description: updates REPO1 REPO2... as defd in /etc/yum/vars/EPAGES_*
operating system: does not work on debian/ubuntu
example: update_yum_repo setup update empty
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local EPAGES_VERSION=yum
  local EPAGES_REPO=ftp://epages-software.de/repo/epages
  local CUSTOMIZED
  if [[ -s /etc/yum/vars/EPAGES_VERSION ]] ; then
    EPAGES_VERSION=$(cat /etc/yum/vars/EPAGES_VERSION)
    CUSTOMIZED=1
  fi
  if [[ -s /etc/yum/vars/EPAGES_REPO ]] ; then
    EPAGES_REPO=$(cat /etc/yum/vars/EPAGES_REPO)
    CUSTOMIZED=1
  fi

  [[ -z $CUSTOMIZED ]] && return

  local file url
  for i in $@ ; do
    file=/etc/yum.repos.d/epages-$i.repo
    [[ -f $file ]] || continue
    for section in $($INI_CONFIG -f $file -G) ; do
      # read templateurl
      url=$($INI_CONFIG -f $file -s $section -k templateurl -G)
      if [[ -z $url ]] ; then
        # if no templateurl found use defaults:
        case $section in
          epages-arch) url=$EPAGES_REPO/kernel/$EPAGES_ARCH ;;
          epages-arch-kernel) url=$EPAGES_REPO/kernel/$EPAGES_KERNEL ;;
          epages-base) url=$EPAGES_REPO/$EPAGES_VERSION/base/$EPAGES_KERNEL ;;
          epages-empty) url=$EPAGES_REPO/$EPAGES_VERSION/empty ;;
          epages-setup) url=$EPAGES_REPO/$EPAGES_VERSION/$EPAGES_ARCH ;;
          epages-update) url=$EPAGES_REPO/$EPAGES_VERSION/update ;;
        esac
      fi
      [[ -z $url ]] && continue
      # set EPAGES_REPO and EPAGES_VERSION
      url=$(eval echo $url)
      # write baseurl
      $INI_CONFIG -f $file -s $section -k baseurl -v $url -i
    done
    # if customized don't use mirrorlist
    sed -ri 's,^([^#].*irrorlist.*),#\1,' $file
  done
}

#..............................................................................
# enable_repo
#..............................................................................
enable_repo() {
  local USAGE='
syntax: enable_repo [-e] {0|1} REPO1 REPO2...
description: enable (1) or disable (0) REPO1 REPO2... (with perl regexp)
options:
     -e: prepend epages- to each REPO (shorthand for epages- repos)
returns: changed repos
operating system: does not work on debian/ubuntu
example: enable_repo -e 0 empty update >/dev/null
'
  [[ $# -lt 2 ]] && { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local file section value arg prefix
  [[ $1 = -e ]] && { prefix=epages-; shift; }
  local ENABLE=$1 ; shift
  [[ $ENABLE = 1 ]] && arg=-e || arg=-d
  local PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

  pushd /etc/yum.repos.d >/dev/null 2>&1 || return
  local CHANGED=$(echo "$@" | tr ' ' "\n" | while read i ; do
    grep -P '\['$prefix$i'\]' $prefix*.repo 2>/dev/null | while IFS=: read file section ; do 
      [[ -z $section ]] && continue
      section=${section//[\[\]]/}
      value=$($INI_CONFIG -f $file -s $section -k enabled -G)
      [[ -z $value ]] && continue
      [[ $value = $ENABLE ]] && continue

      [[ ${EPAGES_KERNEL%s} = $EPAGES_KERNEL ]] || zypper mr $arg $section

      $INI_CONFIG -f $file -s $section -k enabled -v $ENABLE -i
      echo -n "${section#$prefix} "
    done
  done)
  [[ -n $CHANGED ]] && echo $CHANGED
  popd >/dev/null
}

#..............................................................................
# delete_obsolete_repos
#..............................................................................
delete_obsolete_repos() {
  local USAGE='
syntax: delete_obsolete_repos
description: deletes epages repos not necessary anymore
operating system: does not work on debian/ubuntu
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  pushd /etc/yum.repos.d >/dev/null 2>&1 || return
  rm -f mirrors-epages{,-kernel-*,-arch-*-*}
  rm -f epages{,-dev,-archive,-update-archive,-kernel}.repo{,.rpm*}
  popd >/dev/null
}

#..............................................................................
# link_yum2zypper
#..............................................................................
link_yum2zypper() {
  local USAGE='
syntax: link_yum2zypper
description: add linked yum repos to zypper
operating system: works only on SuSE
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  pushd /etc/zypp/repos.d >/dev/null 2>&1 || return
  for i in /etc/yum.repos.d/epages*.repo ; do
    [[ -e "/etc/yum.repos.d/$i.repo" ]] && i="/etc/yum.repos.d/$i.repo"
    [[ ! -f "$i" || $(find "${i##*/}" -printf "%l" 2>/dev/null) = "$i" ]] && continue
    rm -f ${i##*/} 2>/dev/null
    ln -s $i .
  done
  popd >/dev/null
}

#..............................................................................
# unlink_yum2zypper
#..............................................................................
unlink_yum2zypper() {
  local USAGE='
syntax: unlink_yum2zypper
description: remove linked yum repos from zypper
operating system: works only on SuSE
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  pushd /etc/zypp/repos.d >/dev/null 2>&1 || return
  for i in $(find * -type l 2>/dev/null) ; do
    l=$(find "$i" -printf "%l")
    [[ ${l#/etc/yum.repos.d/} = $l ]] && continue
    rm -f $i
  done
  popd >/dev/null
}

#..............................................................................
# refresh_repo_by_upgrade_path
#..............................................................................
refresh_repo_by_upgrade_path() {
  local USAGE='
syntax: refresh_repo_by_upgrade_path [-b TARGET_VERSION] [list of epages version numbers]
description: refresh the repository so that it obeys the upgrade path
returns: shell script that updates the repository database (if necessary)
example: refresh_repo_by_upgrade_path -b 6.17.14 6.17.9 6.17.99
'
  # if epages-common is not installed do nothing
  pkg_installed common || return

  local INSTALLED_VERSION=$(get_cartridge_fullversion Core)
  INSTALLED_VERSION=${INSTALLED_VERSION//[^0-9.]/}
  INSTALLED_VERSION=${INSTALLED_VERSION:-0}

  # repo we want to write into /etc/yum.repos.d/epages-*.repo
  local SET_REPO=${EPAGES_REPO:-http://www.epages.com/repo/epages}
  # existing repo found in /etc/yum.repos.d/epages-*.repo
  local GET_REPO=$(get_arch_repo)
  # for debian/ubuntu we use get_update_repo
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || GET_REPO=$(get_update_repo)

  local PATCH_VERSION REPO_UP_TO_DATE
  local STEP_NUMBER=0
  local TARGET_VERSION STARTING_WITH
  [[ $1 = -b ]] && { TARGET_VERSION=$2 ; shift 2; }
  local HELP_SECTION='Debian_Patching_with_APT_Repository'

  if [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
    # for redhat change HELP_SECTION
    HELP_SECTION='Red_Hat_Patching_with_YUM_Repository'
  else
    # for debian refine the comment
    STARTING_WITH=" (starting with 'apt-get -y update')"
    STARTING_AFTER=" (starting after 'apt-get -y update')"
  fi

  # target version is (name:)?number
  local TARGET_VERSION_NUMBER=${TARGET_VERSION:-9999}
  TARGET_VERSION_NUMBER=${TARGET_VERSION_NUMBER##*:}
  for PATCH_VERSION in $@ ; do
    # die if argument is not a version string
    [[ -n ${PATCH_VERSION//[0-9.]/} ]] && { echo "$USAGE" 1>&2; return 1; }
    # check next argument if patch version is already installed
    test_version $INSTALLED_VERSION -ge $PATCH_VERSION && continue
    test_version $TARGET_VERSION_NUMBER -le $PATCH_VERSION && continue
    STEP_NUMBER=$((STEP_NUMBER + 1))

    # if repo is up-to-date we just show intermediate step to execute
    if [[ -n $REPO_UP_TO_DATE ]] ; then
      echo "
$STEP_NUMBER. To update to ePages $PATCH_VERSION run:
  /var/epages/SetRepository.sh -b $PATCH_VERSION
and re-run the commands as described in$STARTING_AFTER
  http://helpcenter.epages.com/Doc/patch.html#$HELP_SECTION" 1>&2
      continue
    fi

    # only for redhat: return shell script that updates the repository database
    if [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
      echo '
if [ -f /etc/redhat-release ] ; then
  yum --enablerepo="epages-*" --disablerepo "epages*-6.*" clean metadata 2>/dev/null
elif [ -f /etc/SuSE-release ] ; then
  ( zypper clean -m ; zypper refresh ) 2>/dev/null
elif [ -f /etc/debian_version ] ; then
  apt-get -y update
fi'
    fi
    REPO_UP_TO_DATE=1
    # show initial step to execute
    echo "
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
This installation must be updated in various steps:

$STEP_NUMBER. To update to ePages $PATCH_VERSION run the commands as described in$STARTING_WITH
  http://helpcenter.epages.com/Doc/patch.html#$HELP_SECTION" 1>&2

    # check if /etc/yum.repos.d/epages-*.repo is up-to-date
    [[ $GET_REPO = ${GET_REPO#*/$PATCH_VERSION[/} ]] || continue

    # if repo does not end in /epages set it to http://www.epages.com/repo/epages
    [[ $SET_REPO = ${SET_REPO%/epages} ]] && SET_REPO=http://www.epages.com/repo/epages
    # update repo
    /var/epages/SetRepository.sh -r "$SET_REPO" -b $PATCH_VERSION -X -i
  done

  # return if there are no steps to execute
  [[ $STEP_NUMBER = 0 ]] && return
  STEP_NUMBER=$((STEP_NUMBER + 1))

  if [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
    # target version is (name:)?number
    TARGET_VERSION=${TARGET_VERSION%:*}
    # for redhat set TARGET_VERSION=yum as default
    TARGET_VERSION=${TARGET_VERSION:-yum}
  fi

  # show final step to execute
  if [[ -n $TARGET_VERSION ]] ; then
    echo "
$STEP_NUMBER. To update to latest ePages version run:
  /var/epages/SetRepository.sh -b $TARGET_VERSION
and re-run the commands as described in$STARTING_AFTER
  http://helpcenter.epages.com/Doc/patch.html#$HELP_SECTION" 1>&2
  fi
  echo -e "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n" 1>&2
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
