#!/bin/env bash

BN=${0##*/}
DN=$(cd $(dirname $0) && pwd)

usage() {
  ( echo "Usage:"
    echo "  EPAGES_PROJECT=epages6 $BN -d PATCH_DIR [-i INITIAL_VERSION] [-R]"
    echo "Options:"
    echo "  EPAGES_PROJECT: envvar of the epages project (default: epages6)"
    echo "  -d: directory with patch.conf"
    echo "  -i: initial version as e.g. 6013000000"
    echo "  -R: restore" ) 1>&2
  exit 1
}

[[ $# -eq 0 ]] && usage

unset PATCH_DIR RESTORE INITIAL_VERSION
while getopts d:p:i:R c ; do
  case $c in
    d) PATCH_DIR=$OPTARG ;;
    i) INITIAL_VERSION=$OPTARG ;;
    R) RESTORE=1 ;;
    *) usage ;;
  esac
done

[[ -z $PATCH_DIR ]] && usage

if [[ -n $INITIAL_VERSION && -z $RESTORE ]] ; then
  ( cd "$PATCH_DIR" ; ls -d1 6????????? ; echo $INITIAL_VERSION- ) | sort | while read v ; do
    [[ $v != ${v%-} ]] && { new=1 ; continue; }
    [[ -z $new ]] && continue
    echo "$0 -d $PATCH_DIR/$v"
    $0 -d "$PATCH_DIR"/$v
  done
  exit 0
elif [[ -n $INITIAL_VERSION ]] ; then
  ( cd "$PATCH_DIR" ; ls -d1 6????????? ; echo $INITIAL_VERSION- ) | sort -r | while read v ; do
    [[ $v != ${v%-} ]] && break
    echo "$0 -d $PATCH_DIR/$v -R"
    $0 -d "$PATCH_DIR"/$v -R
  done
  exit 0
fi

cd "$PATCH_DIR" || usage
ls patch.conf >/dev/null || usage

PATH=$DN:$PATH:/var/epages
for i in functions.sh ini-config.sh patch-file.sh patch-default.sh ; do
  type $i >/dev/null || exit $?
done

EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
. functions.sh
ETC_DEFAULT=$(readlink -f $(etc_default))
ETC_INITD=/etc/init.d/$EPAGES_PROJECT

[[ -f $ETC_DEFAULT ]] || { ls -l "$ETC_DEFAULT"; exit 1; } 
. "$ETC_DEFAULT"
[[ -z $EPAGES ]] && { "$BN: shell variable EPAGES empty" ; exit 1; }

HOSTNAME=${HOSTNAME:-$(hostname)}

# ----------------------------------------------------------------------
# target_dir
# ----------------------------------------------------------------------

target_dir () {
  local SOURCE=$1
  local TARGET

  case ${SOURCE%%/*} in
    EPAGES_ETC) TARGET=$(dirname "$EPAGES")/etc ;;
    EPAGES_RELEASE) TARGET=/ ;;
    ETC_HTTPD_CONFD) TARGET=$(httpd_confd epages) ;;
    *) TARGET=$(eval echo '$'${SOURCE%%/*}) ;;
  esac

  [[ ${SOURCE#*/} != $SOURCE ]] && TARGET="$TARGET/${SOURCE#*/}"
  echo $TARGET
}


# ----------------------------------------------------------------------
# restore files
# ----------------------------------------------------------------------

if [[ -n $RESTORE ]] ; then

  ini-config.sh -f patch.conf -g | while read SOURCE ; do
    TARGET=$(target_dir "$SOURCE")
    [[ -z $TARGET ]] && continue
    [[ $TARGET = / ]] && TARGET=

    ini-config.sh -f patch.conf -s "$SOURCE" -g | while read SETTINGS ; do
      TARGET_FILE=$TARGET/"${SETTINGS%%=*}"
      [[ ! -f "backup/$HOSTNAME/$TARGET_FILE" ]] && continue
      [[ -f "$TARGET_FILE" ]] && EXISTS=1 || EXISTS=
      echo "backup/$HOSTNAME/$TARGET_FILE -> $TARGET_FILE"
      cat "backup/$HOSTNAME/$TARGET_FILE" > $TARGET_FILE || exit 1
      touch changed.~$$~
      [[ -n $EXISTS ]] && chmod 755 $TARGET_FILE
    done
  done
  [[ -f changed.~$$~ ]] && /etc/init.d/$EPAGES_PROJECT perm admin
  rm -f changed.~$$~

  exit 0
fi

# ----------------------------------------------------------------------
# patch files
# ----------------------------------------------------------------------

ini-config.sh -f patch.conf -g | while read SOURCE ; do
  TARGET=$(target_dir "$SOURCE")
  [[ -z $TARGET ]] && continue
  [[ $TARGET = / ]] && TARGET=

  ini-config.sh -f patch.conf -s "$SOURCE" -g | while read SETTINGS ; do
    SOURCE_FILE=$SOURCE/"${SETTINGS%%=*}"
    TARGET_FILE=$TARGET/"${SETTINGS%%=*}"
    METHOD="${SETTINGS#*=}"
    [[ "${SETTINGS%%=*}" = BASE ]] && continue

    # method = del
    if [[ $METHOD = del ]] ; then
      [[ -f $TARGET_FILE ]] || continue
      mkdir -m 755 -p backup/$HOSTNAME
      cp -Lp --parents $TARGET_FILE backup/$HOSTNAME
      rm -vf $TARGET_FILE
      continue
    fi

    OLD_MD5=$(md5sum "$TARGET_FILE" 2>/dev/null | awk '{print $1}')
    [[ $METHOD = patch && -z $OLD_MD5 ]] && continue # file not found

    # method = patch-file.sh
    if [[ $METHOD != patch && $METHOD != add ]] ; then
      [[ -f $TARGET_FILE ]] || continue
      SOURCE_FILE=file.~$$~
      patch-file.sh -f $TARGET_FILE -p $METHOD > $SOURCE_FILE
    fi

    [[ -f $SOURCE_FILE ]] || continue

    # method = add/patch
    TARGET_DIR=$(dirname $TARGET_FILE)
    [[ -d $TARGET_DIR ]] || mkdir -m 755 -p $TARGET_DIR

    if [[ -n $OLD_MD5 ]] ; then
      NEW_MD5=$(md5sum "$SOURCE_FILE" | awk '{print $1}')
      [[ $NEW_MD5 = $OLD_MD5 && ! $SOURCE_FILE =~ EPAGES_ETC/ ]] && continue
      mkdir -m 755 -p backup/$HOSTNAME
      cp -Lp --parents $TARGET_FILE backup/$HOSTNAME
    fi

    if [[ $METHOD != patch && $METHOD != add ]] ; then
      echo "patch-file.sh -f $TARGET_FILE -p $METHOD > $SOURCE_FILE"
    fi
    echo "$SOURCE_FILE -> $TARGET_FILE"
    cat $SOURCE_FILE > $TARGET_FILE || exit 1
    touch changed.~$$~
    [[ -z $OLD_MD5 ]] && chmod 755 $TARGET_FILE

    # source file = EPAGES_ETC/*/epages6.ept
    unset ETC_FILE
    [[ $SOURCE_FILE = EPAGES_ETC/default/epages6.ept ]] && ETC_FILE="$ETC_DEFAULT"
    [[ $SOURCE_FILE = EPAGES_ETC/init.d/epages6.ept ]] && ETC_FILE="$ETC_INITD"

    if [[ -n $ETC_FILE ]] ; then
      OLD_MD5=$(md5sum "$ETC_FILE" | awk '{print $1}')
      if [[ $SOURCE_FILE = EPAGES_ETC/default/epages6.ept ]] ; then
        echo "patch-default.sh -s $ETC_FILE -t $SOURCE_FILE > file.~$$~"
        patch-default.sh -s "$ETC_FILE" -t $SOURCE_FILE > file.~$$~
      else
        echo "$SOURCE_FILE -> file.~$$~"
        sed "s,_EPAGES_PROJECT_,$EPAGES_PROJECT,g" $SOURCE_FILE > file.~$$~
      fi
      SOURCE_FILE=file.~$$~
      NEW_MD5=$(md5sum "$SOURCE_FILE" | awk '{print $1}')
      [[ $NEW_MD5 = $OLD_MD5 ]] && continue
      mkdir -m 755 -p backup/$HOSTNAME
      cp -Lp --parents "$ETC_FILE" backup/$HOSTNAME
      echo "$SOURCE_FILE -> $ETC_FILE"
      cat $SOURCE_FILE > "$ETC_FILE" || exit 1
    fi

  done
done

if [[ -f changed.~$$~ ]] ; then
  /etc/init.d/$EPAGES_PROJECT perm admin
  if [[ -d $EPAGES_PERL/ ]] ; then
    chmod -R 755 $EPAGES_PERL/bin $EPAGES_PERL/lib32 2>/dev/null
    chown -R ep_appl:epages $EPAGES_PERL/bin $EPAGES_PERL/lib32 2>/dev/null
  fi
fi

rm -f file.~$$~ changed.~$$~

exit 0
