#!/bin/bash

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo "Error: $@"
  echo "Usage:"
  echo "  $0 PACKAGE [REQ_VERSION]"
  echo "Description:"
  echo "  Checks if PACKAGE is installed [with at least REQ_VERSION]"
  echo "  PACKAGE must start with letter/digit, REQ_VERSION with digit."
  echo "  Echoes the latest installed version."
  echo "Returns success if:"
  echo "  - REQ_VERSION is not defined and PACKAGE is installed"
  echo "  - REQ_VERSION is defined and PACKAGE is not installed"
  echo "  - REQ_VERSION is defined and installed PACKAGE is up-to-date"
  echo "Otherwise returns failure."
  echo "Examples:"
  echo "  $0 make 3.80"
  echo "  $0 epages-release 6.17.5"
  echo "  $0 epages-webconf"
  exit 1
}

PACKAGE=$1
REQ_VERSION=$2

# script must have 1..2 arguments
[[ $# -eq 0 || $# -gt 2 ]] && usage

if [[ ${PACKAGE#[0-9a-zA-Z]} = $PACKAGE ]] ; then
  usage "PACKAGE=$PACKAGE must start with letter/digit"
fi
if [[ -n $REQ_VERSION && ${REQ_VERSION#[0-9]} = $REQ_VERSION ]] ; then
  usage "REQ_VERSION=$REQ_VERSION must start with digit"
fi

#------------------------------------------------------------------------------
# execute
#------------------------------------------------------------------------------

. /var/epages/functions.sh

case $EPAGES_KERNEL in
  *d) # debian
      dpkg --status $PACKAGE >/dev/null 2>&1 ;;
  *)  # redhat/suse
      rpm --quiet -q $PACKAGE >/dev/null 2>&1 ;;
esac

[[ $? = 0 ]] && FOUND=1 || FOUND=
unset LATEST_VERSION FOUND_VERSION

if [[ -n $FOUND ]] ; then
  # create tempdir
  mkdir /tmp/${0##*/}.~$$~
  pushd /tmp/${0##*/}.~$$~ >/dev/null || exit 1
  # and add empty files with name is found version
  case $EPAGES_KERNEL in
    *d) dpkg-query --show -f '${Version}\n' $PACKAGE ;;
    *)  rpm -q $PACKAGE --qf '[%{VERSION}-%{RELEASE}\n]' ;;
  esac | xargs -i{} touch "{}#"

  # find latest installed version (ls -v sorts by version)
  FOUND_VERSION=$(ls -vr1 | head -1)
  # and add empty file with name is required version  
  [[ -n $REQ_VERSION ]] && touch $REQ_VERSION
  # find latest version (ls -v sorts by version)
  LATEST_VERSION=$(ls -vr1 | head -1)
  popd >/dev/null
  # delete tempdir
  rm -rf /tmp/${0##*/}.~$$~
fi

# echo latest installed version
[[ -n $FOUND_VERSION ]] && echo ${FOUND_VERSION%#}

if [[ -z $REQ_VERSION ]] ; then
  # REQ_VERSION is not defined

  # return success/failure if PACKAGE is installed/missing
  [[ -n $FOUND ]] && exit 0 || exit 1

else
  # REQ_VERSION is defined

  # return success if PACKAGE is not installed
  [[ -z $FOUND ]] && exit 0
  # return failure if version of installed PACKAGE not found
  [[ -z $FOUND_VERSION ]] && exit 1
  # return success/failure if installed PACKAGE is up-to-date/outdated
  [[ $FOUND_VERSION = $LATEST_VERSION ]] && exit 0 || exit 1
fi

# -----------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
exit 0
